<?php
/*
 Plugin Name: Premise
 Plugin URI: http://getpremise.com/
 Description: Quickly and easily create amazing landing pages for your WordPress site.
 Version: 1.1.1
 Author: Copyblogger Media LLC
 Author URI: http://www.copyblogger.com
 */

/** Credits
 *
 * Some TinyMCE icons adapted from the Led Icon set – http://led24.de/iconset/
 */

if(!class_exists('Premise')) {
	class Premise {

		var $_cached_DesignPageSlug = '';
		var $_cached_FrontPageId = null;
		var $_cached_NumberPluginColumns = 1;

		var $_data_EditorNumber = 0;
		var $_data_PremiseApi = null;
		var $_data_TinyMCESelectors = array();
		var $_data_RegisteredLandingPages = array();
		var $_data_LandingPageIds = array(
			'long-copy' => 1,
			'content-landing-page'=> 4,
			'pricing' => 5,
			'opt-in' => 6,
			'video'=> 7,
			'content-scroller' => 9,
			'thank-you' => 12,
		);
		var $_data_PluginSlug = 'getpremise';
		var $_data_PluginInfoURL = 'http://api.getpremise.com/plugin/';
		var $_data_PluginURL = 'http://getpremise.com/';
		var $_data_VersionTransientName = 'premise_version_info';
		var $_data_VersionURL = 'http://api.getpremise.com/version/';
		var $_data_Version = '1.1.1';

		var $_metakey_Duplicated = '_premise_landing_page_duplicated';
		var $_metakey_LandingPageType = '_premise_landing_page_type';
		var $_metakey_LandingPageAdviceType = '_premise_landing_page_advice_type';
		var $_metakey_SeoSettings = '_premise_seo_settings';
		var $_metakey_Settings = '_premise_settings';
		var $_metakey_Skin = '_premise_skin';
		var $_metakey_TrackingSettings = '_premise_tracking_settings';

		var $_optin_AweberApplicationId = '3ca8152d';
		var $_optin_AweberAuthenticationUrl = 'https://auth.aweber.com/1.0/oauth/authorize_app/';
		var $_optin_ConstantContactCachedMessages = null;
		var $_optin_ConstantContactHttpStatuses = array( 200 => 'Success - The request was successful', 201 => 'You have been subscribed.', 400 => 'Invalid Request - There are many possible causes for this error, but most commonly there is a problem with the structure or content of XML your application provided. Carefully review your XML. One simple test approach is to perform a GET on a URI and use the GET response as an input to a PUT for the same resource. With minor modifications, the input can be used for a POST as well.', 401 => 'Unauthorized - This is an authentication problem. Primary reason is that the API call has either not provided a valid API Key, Account Owner Name and Associated Password or the API call attempted to access a resource (URI) which does not match the same as the Account Owner provided in the login credientials.', 404 => 'URL Not Found - The URI which was provided was incorrect. Compare the URI you provided with the documented URIs. Start here.', 409 => 'Conflict - There is a problem with the action you are trying to perform. Commonly, you are trying to "Create" (POST) a resource which already exists such as a Contact List or Email Address that already exists. In general, if a resource already exists, an application can "Update" the resource with a "PUT" request for that resource.', 415 => 'Unsupported Media Type - The Media Type (Content Type) of the data you are sending does not match the expected Content Type for the specific action you are performing on the specific Resource you are acting on. Often this is due to an error in the content-type you define for your HTTP invocation (GET, PUT, POST). You will also get this error message if you are invoking a method (PUT, POST, DELETE) which is not supported for the Resource (URI) you are referencing. To understand which methods are supported for each resource, and which content-type is expected, see the documentation for that Resource.', 500 => 'Server Error', );
		var $_optin_ConstantContactKey = 'a1a2e20e-e5e9-4879-8a20-0339affa0c6a';
		var $_optin_ConstantContactMessages = '_premise_optin_constant_contact_messages';
		var $_optin_ConstantContactNumber = 1;
		var $_optin_Keys = array('aweber' => 'Aweber', 'mailchimp' => 'MailChimp', 'constant-contact' => 'Constant Contact', 'manual' => 'Other (Copy & Paste)');
		var $_optin_ManualBase = '_premise_optin_manual_form_';
		var $_optin_MailChimpCachedMessages = null;
		var $_optin_MailChimpMergeVars = array();
		var $_optin_MailChimpMessages = '_premise_optin_mailchimp_messages';
		var $_optin_MailChimpNumber = 1;

		var $_option_ConfiguredButtons = '_premise_configured_buttons';
		var $_option_PinnedStatus = '_premise_pinned_status';
		var $_option_DesignSettings = '_premise_design_settings';
		var $_option_SettingsTransient = '_premise_settings_transient';

		var $_type_LandingPage = 'landing_page';

		/// INITIALIZERS

		function Premise() {
			$this->addActions();
			$this->addFilters();

			register_activation_hook(__FILE__, array($this, 'activate'));

			$this->initializeApi();
			$this->initializeLandingPages();
		}

		function addActions()  {
			add_action('admin_head', array(&$this, 'processDuplicateDisplay'), 100);
			add_action('admin_init', array(&$this, 'addTinyMCEButtons'));
			add_action('admin_init', array(&$this, 'processSubmissions'));
			add_action('admin_menu', array(&$this, 'addAdministrativeInterfaceItems'));
			add_action('admin_menu', array(&$this, 'modifyAdministrativeInterfaceItems'), 1010);
			add_action('admin_notices', array(&$this, 'addNoticeIfPremiseCannotGenerateCSS'));
			add_action('edit_form_advanced', array(&$this, 'displayPremiseContentScrollerTabs'));
			add_action('edit_form_advanced', array(&$this, 'displayPremisePricingColumns'));
			add_action('init', array(&$this, 'registerShortcodes'));
			add_action('load-update-core.php', array(&$this, 'deleteUpdateTransient'));
			add_action('parse_request', array(&$this, 'processMailingListContactSubmissions'));
			add_action('setup_theme', array(&$this, 'possiblyChangeThemeConstants'), 11);
			add_action('setup_theme', array(&$this, 'registerCustomContentTypes'), 10);
			add_action('manage_posts_custom_column', array(&$this, 'outputLandingPageTypeColumn'), 10, 2);
			add_action('save_post', array(&$this, 'saveLandingPageMeta'), 10, 2);
			add_action('update_option_'.$this->_option_ConfiguredButtons, array(&$this, 'saveConfiguredButtonsStylesheet'), 10, 2);
			add_action('wp_ajax_premise_save_button', array(&$this, 'saveButtonViaAjax'));
			add_action('wp_ajax_premise_use_graphic', array(&$this, 'useGraphic'));
			add_action('wp_ajax_premise_sample_copy', array(&$this, 'retrievePageTypeSampleCopy'));
			add_action('wp_ajax_premise_optin', array(&$this, 'retrieveOptInCode'));
			add_action('wp_ajax_premise_pinned', array(&$this, 'modifyPinnedStatus'));
			add_action('wp_ajax_premise_get_lists', array(&$this, 'ajaxOptinGetLists'));
			add_action('wp_ajax_premise_save_optin_manual', array(&$this, 'ajaxOptinSaveFormCode'));

			/// PLUGIN UPDATES
			add_filter('transient_update_plugins', array($this, 'checkForUpdate'));
			add_filter('site_transient_update_plugins', array($this, 'checkForUpdate'));

			/// PREMISE THEME
			add_action('wp_head', array(&$this, 'outputMetaTagsAsAppropriate'), -1);
			add_action('wp_head', array(&$this, 'addLandingPagesCssAndJs'), 0);
			add_action('wp_head', array(&$this, 'outputGoogleWebsiteOptimizerScriptAsAppropriate'), 9);
			add_action('wp_head', array(&$this, 'outputVisualWebsiteOptimizerScriptAsAppropriate'), 9);
			add_action('premise_immediately_after_head', array(&$this, 'outputGoogleWebsiteOptimizerFooterScriptAsAppropriate'), 9);
			add_action('wp_title', array(&$this, 'modifyWPTitleForLandingPages'), 25);
			add_action('wp_head', array(&$this, 'addLandingPagesScriptsToHeader'), 11);
			add_action('wp_footer', array(&$this, 'addLandingPagesScriptsToFooter'));

			/**
			 * The following actions control the popup thickbox that allows for searching
			 * and using premise resources.
			 */
			add_action('media_upload_premise-button-create', array(&$this, 'displayPremiseResourcesThickbox'));
			add_action('media_upload_premise-button-usage', array(&$this, 'displayPremiseResourcesThickbox'));
			add_action('media_upload_premise-resources-education', array(&$this, 'displayPremiseResourcesThickbox'));
			add_action('media_upload_premise-resources-graphics', array(&$this, 'displayPremiseResourcesThickbox'));
			add_action('media_upload_premise-resources-optin', array(&$this, 'displayPremiseResourcesThickbox'));

			/**
			 * This next little part is totally hacky so that we can
			 * make sure people select a Landing Page Type before a
			 * landing page is actually created.
			 *
			 * This is a limitation of the WordPress plugin system, but keeps
			 * it relatively tucked away so that we can replace it in the
			 * future if necessary.
			 */
			add_action('admin_notices', array(&$this, 'possiblyStartOutputBuffering'));
			add_filter('wp_comment_reply', array(&$this, 'possiblyEndOutputBuffering'));

			/**
			 * Custom Meta Boxes Per Type
			 */
			add_action('register_meta_boxes-content-scroller.php', array(&$this, 'registerContentScrollerTypeMetaBoxes'));
			add_action('register_meta_boxes-landing.php', array(&$this, 'registerLandingTypeMetaBoxes'));
			add_action('register_meta_boxes-long-copy.php', array(&$this, 'registerLongCopyTypeMetaBoxes'));
			add_action('register_meta_boxes-opt-in.php', array(&$this, 'registerOptInTypeMetaBoxes'));
			add_action('register_meta_boxes-pricing.php', array(&$this, 'registerPricingTypeMetaBoxes'));
			add_action('register_meta_boxes-video.php', array(&$this, 'registerVideoTypeMetaBoxes'));
			
			add_action('after_plugin_row', array($this, 'addPluginSupportRow'), 10, 3);
		}

		function addFilters() {
			add_filter('attachment_fields_to_edit', array(&$this, 'modifyAttachmentFields'), 11, 2);
			add_filter('generate_rewrite_rules', array(&$this, 'generateRewriteRules'));
			add_filter('manage_edit-'.$this->_type_LandingPage.'_columns', array(&$this, 'addColumnHeaderForPageType'));
			add_filter('manage_plugins_columns', array($this, 'saveNumberPluginColumns'), 100000);
			add_filter('manage_plugins-network_columns', array($this, 'saveNumberPluginColumns'), 100000);
			add_filter('plugin_action_links_'.plugin_basename(__FILE__), array($this, 'addSupportLinkForPremise'), 10, 4);
			add_filter('network_admin_plugin_action_links_'.plugin_basename(__FILE__), array($this, 'addSupportLinkForPremise'), 10, 4);
			add_filter('plugins_api', array($this, 'overridePluginsAPIResults'), 10, 3);
			add_filter('post_type_link', array(&$this, 'landingPagePermalink'), 10, 3);
			add_filter('premise_get_optin_form_code', 'do_shortcode');
			add_filter('pre_update_option_'.$this->_option_DesignSettings, array(&$this, 'filterDesignSettings'),10,2);
			add_filter('screen_layout_columns', array(&$this, 'modifyLayoutColumns'), 10, 2);
			add_filter('site_transient_theme_roots', array(&$this, 'modifyThemeRootsTransient'));
			add_filter('the_content', array(&$this, 'removeContentFilters'), 7);
			add_filter('wp_head', array(&$this, 'removeOtherDepdendenciesFromPremisePage'), 100);
			add_filter('template_include', array(&$this, 'modifyTemplateInclude'));
			add_filter('tiny_mce_version', array(&$this, 'registerTinyMCEScriptOutput'));
			add_filter('wp_dropdown_pages', array($this,'addPageToReadingStaticPageDropdown'));
		}

		function activate() {
			// TODO RESET DESIGN SETTINGS IF THEY AREN'T PRESENT
		}

		function initializeApi() {
			$settings = $this->getSettings();
			$this->_data_PremiseApi = new Premise_API($settings['main']['api-key']);
		}

		function convertLandingPageToId($advice) {
			return $this->_data_LandingPageIds[$advice];
		}

		function initializeLandingPages() {
			$this->addLandingPageType('long-copy', __('Sales Page'), __('A simple sales page.'), 'long-copy');
			$this->addLandingPageType('long-copy', __('Content Landing Page'), __('Content based landing page.'), 'content-landing-page');
			$this->addLandingPageType('pricing', __('Pricing'), __('Pricing table landing page.'), 'pricing');
			$this->addLandingPageType('opt-in', __('Opt In'), __('Landing page with opt-in form.'), 'opt-in');
			$this->addLandingPageType('video', __('Video'), __('Landing page with video.'), 'video');
			$this->addLandingPageType('content-scroller', __('Tab Scroller'), __('Landing page with multiple tabs on content scroller.'), 'content-scroller');
			$this->addLandingPageType('long-copy', __('Thank You'), __('Say thanks to your customers after they take action.'), 'thank-you');
		}

		function addLandingPageType($template, $name, $description, $advice = '') {
			if(empty($advice)) {
				$advice = $template;
			}

			if(!isset($this->_data_RegisteredLandingPages[$advice])) {
				$data = array('template' => $template, 'name' => $name, 'description' => $description, 'advice' => $advice);
				$this->_data_RegisteredLandingPages[$advice] = $data;
			}
		}

		/// FUNCTIONAL CALLBACKS

		//// OPT IN AJAX

		function ajaxOptinGetLists() {
			$data = stripslashes_deep($_POST);

			$lists = array();
			switch($data['provider']) {
				case 'aweber':
					$lists = $this->getAweberLists();
					break;
				case 'mailchimp':
					$lists = $this->getMailChimpLists();
					break;
				case 'constant-contact':
					$lists = $this->getConstantContactLists();
					break;
			}

			echo json_encode($lists);
			exit();
		}

		function ajaxOptinSaveFormCode() {
			$data = stripslashes_deep($_POST);

			$id = time();
			$key = $this->_optin_ManualBase.$id;

			if(!get_option($key, false)) {
				add_option($key, $data['code']);
				echo json_encode(array('error' => false, 'id' => $id));
			} else {
				echo json_encode(array('error' => true));
			}

			exit();
		}

		//// OTHER
		
		function addPluginSupportRow($pluginFile, $pluginData, $status) {
			$basename = plugin_basename(__FILE__);
			if($pluginFile != $basename) { return; }

			$count = $this->_cached_NumberPluginColumns;
			
			$settings = $this->getSettings();
			$apikey = $settings['main']['api-key'];
			if(empty($apikey)) {
				$valid = false;
			} else {
				$version = $this->getVersionInfo();
				$valid = is_wp_error($version) ? true : 1 == $version->is_valid;
			}
			include('views/misc/plugin-support-row.php');
		}
		
		function addSupportLinkForPremise($actions, $plugin_file, $plugin_data, $context) {
			$actions['support'] = '<a href="#" id="premise-support-dropdown">Get Support</a>';
			return $actions;
		}
		
		function saveNumberPluginColumns($columns) {
			$this->_cached_NumberPluginColumns = count($columns);
			return $columns;
		}

		/**
		 * This callback adds the appropriate settings menu and submenus for Premise
		 * plugin administration.
		 * @return void
		 */
		function addAdministrativeInterfaceItems() {
			$hooks = array('post-new.php','post.php');

			$hooks[] = add_menu_page(__('Premise'), __('Premise'), 'manage_options', 'premise-main', array(&$this, 'displayPremiseSettingsPage'), plugins_url('resources/images/icon.gif', __FILE__), '56.6');

			$hooks[] = add_submenu_page('premise-main', __('Main Settings'), __('Main Settings'), 'manage_options', 'premise-main', array(&$this, 'displayPremiseSettingsPage'));
			
			$hooks[] = add_submenu_page('premise-main', __('Style Settings'), __('Style Settings'), 'manage_options', 'premise-styles', array(&$this, 'displayPremiseSettingsPage'));
			
			$styleTitle = __('Add Style');
			if('premise-style-settings' == $_GET['page'] && $this->isValidStyleKey($_GET['premise-design-key'])) {
				$styleTitle = __('Edit Style');	
			}
			
			$design = add_submenu_page('premise-main', $styleTitle, $styleTitle, 'manage_options', 'premise-style-settings', array(&$this, 'displayPremiseSettingsPage'));
			$this->_cached_DesignPageSlug = $design;
			$hooks[] = $design;

			add_action("load-{$design}", array(&$this, 'settingsBoxes'));
			
			$hooks[] = $premiseHelp = add_submenu_page('premise-main', __('Premise Help'), __('Premise Help'), 'manage_options', 'premise-help', array(&$this, 'displayPremiseSettingsPage'));
			add_action("load-{$premiseHelp}", array(&$this, 'redirectToPremiseHelpPage'));

			foreach($hooks as $hook) {
				add_action("admin_print_styles-{$hook}", array(&$this, 'enqueueAdministrativeResourcesForAdminPages'));
			}

			add_action('admin_enqueue_scripts', array(&$this, 'possiblyEnqueueAdministrativeResourcesForAdminPages'));

			add_action("admin_print_styles-{$design}", array(&$this, 'enqueueAdministrativeResourcesForDesignPage'));
		}

		function addColumnHeaderForPageType($columns) {
			$new = array('landing-page-type'=>__('Landing Page Type'));
			$columns = array_slice($columns, 0, 2, true) + $new + array_slice($columns, 2, count($columns), true);
			return $columns;
		}

		function addLandingPagesCssAndJs() {
			global $wp, $wp_query;
			if(is_singular(array($this->_type_LandingPage)) || $wp->query_vars['post_type'] == $this->_type_LandingPage) {
				$meta = $this->getPremiseMeta($wp_query->get_queried_object_id());
				$key = $meta['style'];
				
				if(!file_exists(premise_get_settings_stylesheet_path($key)) || trim(premise_get_settings_stylesheet_contents($key)) == '') {
					premise_create_stylesheets();
				}
				
				if(!file_exists(premise_get_custom_buttons_stylesheet_path())) {
					$this->saveConfiguredButtonsStylesheet(array(), $this->getConfiguredButtons());
				}
				
				if ( !premise_is_minified($key) && file_exists(premise_get_settings_stylesheet_path($key)) ) {
					wp_enqueue_style('premise', plugins_url('themes/premise/style.css', __FILE__), array(), filemtime(dirname(__FILE__).'/themes/premise/style.css'));
					wp_enqueue_style('premise_settings_stylesheet', premise_get_settings_stylesheet_url($key), false, filemtime(premise_get_settings_stylesheet_path($key)));
				} elseif (premise_is_minified($key) && file_exists(premise_get_minified_stylesheet_path($key)) ) {
					// Otherwise, if minified, then add reference to minified stylesheet, and remove style.css reference
					wp_enqueue_style('premise_minified_stylesheet', premise_get_minified_stylesheet_url($key), false, filemtime(premise_get_minified_stylesheet_path($key)));
				}
				
				wp_enqueue_style('premise_custom_buttons', premise_get_custom_buttons_stylesheet_url());
				wp_enqueue_script('premise_easing', plugins_url('themes/premise/js/jquery-easing.js', __FILE__), array('jquery'));
				wp_enqueue_script('premise_coda_slider', plugins_url('themes/premise/js/jquery-coda.js', __FILE__), array('jquery', 'premise_easing'));
				wp_enqueue_script('premise_pretty_photo', plugins_url('themes/premise/js/jquery-overlay.js', __FILE__), array('jquery'));
			}
		}

		function addLandingPagesScriptsToHeader() {
			global $wp;
			if(is_singular(array($this->_type_LandingPage)) || $wp->query_vars['post_type'] == $this->_type_LandingPage) {
				$settings = $this->getSettings();
				if(!empty($settings['scripts']['header'])) {
					echo "\n{$settings['scripts']['header']}\n";
				}
				
				global $wp_query;
				$meta = $this->getPremiseMeta($wp_query->get_queried_object_id());
				$meta['header-scripts'] = trim($meta['header-scripts']);
				if(!empty($meta['header-scripts'])) {
					echo "\n{$meta['header-scripts']}\n";
				}
			}
		}

		function addLandingPagesScriptsToFooter() {
			global $wp;
			if(is_singular(array($this->_type_LandingPage)) || $wp->query_vars['post_type'] == $this->_type_LandingPage) {
				$settings = $this->getSettings();
				if(!empty($settings['scripts']['footer'])) {
					echo "\n{$settings['scripts']['footer']}\n";
				}
				
				global $wp_query;
				$meta = $this->getPremiseMeta($wp_query->get_queried_object_id());
				$meta['footer-scripts'] = trim($meta['footer-scripts']);
				if(!empty($meta['footer-scripts'])) {
					echo "\n{$meta['footer-scripts']}\n";
				}
			}
		}

		function addNoticeIfPremiseCannotGenerateCSS() {
			$directory = premise_get_stylesheet_location('path');
			if(!file_exists($directory)) {
				@mkdir($directory, 0755, true);
			}
			if($_GET['page'] == 'premise-style-settings' && !is_writeable($directory)) {
				include('views/misc/not-writeable-notice.php');
			}
			
			if($_GET['premise-sent-support-request'] == 'yes') {
				include('views/misc/sent-support-request-admin-notice.php');
			}
		}

		function addPageToReadingStaticPageDropdown($page_html) {
			if ('page_on_front' != substr($page_html, 14, 13)) {
				return $page_html;				
			}
			
			$selected = false;
			$page_html = str_replace( '</select>', '', $page_html );
			$selected = get_option('page_on_front');
			
			$landings = get_posts(array('post_type' => $this->_type_LandingPage, 'nopaging' => true, 'post_status' => 'publish'));
			foreach($landings as $landing) {
				$page_html .= sprintf('<option %s class="level-0" value="%d">%s</option>%s', $selected == $landing->ID ? 'selected="selected"' : '', $landing->ID, apply_filters('the_title', $landing->post_title), "\n");
			}
			$page_html .= '</select>';
			return $page_html;
		}

		function addTinyMCEButtons() {
			if(user_can_richedit()) {
				add_filter("mce_external_plugins", array(&$this, 'addPremiseTinyMCEPlugin'));
				add_filter('mce_buttons_3', array(&$this, 'addPremiseTinyMCEPluginButtons'));
			}
		}

		function addPremiseTinyMCEPlugin($plugin_array) {
			global $pagenow, $post;
			if(in_array($pagenow, array('post.php','post-new.php')) && $post->post_type == $this->_type_LandingPage) {
				$plugin_array['PremiseInfo'] = plugins_url('resources/tmce/editor_plugin.js', __FILE__);
			}
			return $plugin_array;
		}

		function addPremiseTinyMCEPluginButtons($buttons) {
			global $pagenow, $post;
			if(in_array($pagenow, array('post.php','post-new.php')) && $post->post_type == $this->_type_LandingPage) {
				array_push($buttons, 'PremiseSampleCopy');
				array_push($buttons, 'PremiseInsertGraphic');
				array_push($buttons, 'PremiseInsertOptIn');
				array_push($buttons, 'PremiseInsertNoticeBox');
				array_push($buttons, 'PremiseInsertButton');
				array_push($buttons, '|');
				array_push($buttons, 'PremiseInsertGWOSection');
				array_push($buttons, 'PremiseGWOAddConversionLink');
				array_push($buttons, 'PremiseGWORemoveConversionLink');

			}
			return $buttons;
		}

		function addTinyMCEToPremiseEditor() {
			if(!$this->_data_PreviouslyOutputTinyMCEScriptTags) {
				wp_tiny_mce();
			}
			include('views/misc/tiny-mce-append.php');
		}

		function aweberFormShortcode($atts, $content = null) {
			$atts = shortcode_atts(array('id' => '', 'align' => 'none', 'title' => __('Sign Up')), $atts);
			$title = $atts['title'];
			if(empty($atts['id'])) {
				return '';
			}

			if(is_ssl()) {
				$src = 'https://';
			} else {
				$src = 'http://';
			}
			$id = absint($atts['id']);
			$src .= sprintf('forms.aweber.com/form/%d/%d.js', $id % 100, $id);
			$script = sprintf('<script type="text/javascript" src="%s"></script>', $src);

			ob_start();
			include('views/misc/aweber-contact-form.php');
			return ob_get_clean();
		}

		function checkForUpdate($option) {
			$info = $this->getVersionInfo();

			if(!$info) {
				return $option;
			}

			$plugin = plugin_basename(__FILE__);
			if(!is_object($option->response[$plugin])) {
				$option->response[$plugin] = new stdClass();
			}

			//Empty response means that the key is invalid. Do not queue for upgrade
			if(!$info->is_valid || version_compare($this->_data_Version, $info->version, '>=')) {
				unset($option->response[$plugin]);
			} else {
				$settings = $this->getSettings();
				$option->response[$plugin]->url = $this->_data_PluginURL;
				$option->response[$plugin]->slug = $this->_data_PluginSlug;;
				$option->response[$plugin]->package = add_query_arg(array('apikey'=>$settings['main']['api-key']), $info->url);
				$option->response[$plugin]->new_version = $info->version;
				$option->response[$plugin]->id = "0";
			}

			return $option;
		}

		function constantContactFormShortcode($atts, $content = null) {
			$atts = shortcode_atts(array('id' => '', 'align' => 'none', 'title' => __('Sign Up')), $atts);
			$title = $atts['title'];
			if(empty($atts['id'])) {
				return '';
			}

			$id = absint($atts['id']);
			$align = $atts['align'];

			if(!in_array($align, array('none','center','left','right'))) {
				$align = 'none';
			}

			if(!is_array($this->_optin_ConstantContactCachedMessages)) {
				$this->_optin_ConstantContactCachedMessages = get_transient($this->_optin_ConstantContactMessages);
				delete_transient($this->_optin_ConstantContactMessages);
				if(!is_array($this->_optin_ConstantContactCachedMessages)) {
					$this->_optin_ConstantContactCachedMessages = array();
				}
			}

			$messages = $this->_optin_ConstantContactCachedMessages;

			$ccnumber = $this->_optin_ConstantContactNumber;
			$this->_optin_ConstantContactNumber++;

			ob_start();
			include('views/misc/constant-contact-form.php');
			return ob_get_clean();
		}

		function deleteUpdateTransient() {
			delete_transient($this->_data_VersionTransientName);
		}

		/**
		 * This callbacks ensures that the Premise specific CSS and JS are loaded on the appropriate pages.
		 * @return void
		 */
		function enqueueAdministrativeResourcesForAdminPages() {
			global $pagenow, $post;
			if(in_array($pagenow, array('post.php', 'post-new.php')) && $post->post_type != $this->_type_LandingPage) {
				return;
			}

			global $editor_styles;
			$editor_styles = array();

			wp_enqueue_style('premise_custom_buttons', premise_get_custom_buttons_stylesheet_url());
			wp_enqueue_style('premise-admin', plugins_url('resources/premise-admin.css',__FILE__), array('thickbox'));
			wp_enqueue_script('premise-admin', plugins_url('resources/premise-admin.js',__FILE__), array('jquery', 'editor', 'thickbox'));
			wp_localize_script('premise-admin', 'Premise', array('tabs_warning'=>5, 'graphics_title' => __('Premise Graphics'), 'optin_title' => __('Premise Opt In'), 'button_title' => __('Premise Buttons')));
		}

		function enqueueAdministrativeResourcesForDesignPage() {
			wp_enqueue_style('farbtastic');

			wp_enqueue_script('common');
			wp_enqueue_script('wp-lists');
			wp_enqueue_script('postbox');
			wp_enqueue_script('farbtastic');
			wp_enqueue_script('premise-design', plugins_url('resources/premise-design.js', __FILE__), array('farbtastic', 'thickbox'));
			$params = array( 'pageHook' => $this->_cached_DesignPageSlug, 'firstTime' => !is_array(get_user_option('closedpostboxes_'.$this->_cached_DesignPageSlug)), 'toggleAll' => __('Toggle All'), 'warnUnsaved' => __('The changes you made will be lost if you navigate away from this page.'), 'warnReset' => __('Are you sure you want to reset?') );
			wp_localize_script('premise-design', 'PremiseDesign', $params);
		}

		function filterDesignSettings($newvalue, $oldvalue) {
			$defaults = $this->getDefaultDesignSettings();
			foreach ($newvalue as $akey => $array) {
				if(is_array($array)) {
					foreach($array as $key => $value) {
						if ( '' == $value // Empty
						|| (preg_match('/_color$/', $key) && 0 == preg_match('/^#(([a-fA-F0-9]{3}$)|([a-fA-F0-9]{6}$))/', $value)) // Invalid color code
						|| ((strpos($key, '_size') || strpos($key, '_height') || strpos($key, '_margin') || strpos($key, '_padding') || strpos($key, '_radius') || strpos($key, '_width') || preg_match('/_border$/', $key))
						&& !ctype_digit($value)) // Not a digit
						) {
							$newvalue[$akey][$key] = $defaults[$key]; // Save default value instead.
						}
					}
				}
			}
			return $newvalue;
		}

		function generateRewriteRules($wp_rewrite) {
			$settings = $this->getSettings();

			$rules = array();
			if($settings['main']['rewrite-root'] == 1 || empty($settings['main']['rewrite'])) {
				$uris = $this->getLandingPageUris();
				foreach($uris as $uri) {
					$rules["{$uri}/?$"] = 'index.php?post_type='.$this->_type_LandingPage.'&name='.$uri;
				}
			} else {
				$rules[$settings['main']['rewrite'].'/(.+)/?$'] = 'index.php?post_type='.$this->_type_LandingPage.'&name='.$wp_rewrite->preg_index(1);
			}

			$wp_rewrite->rules = $rules + $wp_rewrite->rules;
		}

		function getPremiseThickboxTabs($vals) {
			return array('premise-resources-education'=>__('Education'), 'premise-resources-graphics'=>__('Graphics'));
		}

		function gwoSectionShortcode($atts, $content = null) {
			if(empty($content)) {
				return '';
			}

			$atts = shortcode_atts(array('id'=>''), $atts);
			if(empty($atts['id'])) {
				return $content;
			} else {
				return '<script>utmx_section("'.esc_attr($atts['id']).'")</script>'.$content.'</noscript>';
			}

			return $content;
		}

		function landingPagePermalink($link, $post, $leavename) {
			if($post->post_type == $this->_type_LandingPage) {
				if('' == get_option('permalink_structure')) {
					$link = add_query_arg(array('post_type' => $this->_type_LandingPage, 'p' => $post->ID), home_url('/'));
				} else {
					$settings = $this->getSettings();
					if($settings['main']['rewrite-root'] || empty($settings['main']['rewrite'])) {
						$link = untrailingslashit(home_url("/%landing_page%/"));
					} else {
						$link = untrailingslashit(home_url("/{$settings['main']['rewrite']}/%landing_page%/"));
					}

					if(!$leavename) {
						if(empty($post->post_name)) {
							$link = add_query_arg(array('post_type' => $this->_type_LandingPage, 'p' => $post->ID), home_url('/'));
						} else {
							$link = str_replace('%landing_page%', $post->post_name, $link);
						}
					}
				}
			}

			return $link;
		}

		function mailchimpFormShortcode($atts, $content = null) {
			$atts = shortcode_atts(array('id' => '', 'align' => 'none', 'title' => __('Sign Up')), $atts);
			$title = $atts['title'];
			if(empty($atts['id'])) {
				return '';
			}

			$id = $atts['id'];
			$align = $atts['align'];

			if(!in_array($align, array('none','center','left','right'))) {
				$align = 'none';
			}

			if(!is_array($this->_optin_MailChimpCachedMessages)) {
				$this->_optin_MailChimpCachedMessages = get_transient($this->_optin_MailChimpMessages);
				delete_transient($this->_optin_MailChimpMessages);
				if(!is_array($this->_optin_MailChimpCachedMessages)) {
					$this->_optin_MailChimpCachedMessages = array();
				}
			}

			$messages = $this->_optin_MailChimpCachedMessages;

			$mcnumber = $this->_optin_MailChimpNumber;
			$this->_optin_MailChimpNumber++;

			$mv = $this->getMailChimpMergeVars($id);
			if(!$mv) {
				return '';
			}


			ob_start();
			include('views/misc/mailchimp-form.php');
			return ob_get_clean();
		}

		function manualOptInFormShortcode($atts, $content = null) {
			$atts = shortcode_atts(array('id' => '', 'title' => __('Sign Up')), $atts);
			if(empty($atts['id'])) {
				return '';
			}

			$title = $atts['title'];

			$key = $this->_optin_ManualBase.$atts['id'];
			$script = get_option($key);
			$script = do_shortcode($script);
			ob_start();
			include('views/misc/manual-contact-form.php');
			return ob_get_clean();
		}

		function modifyAdministrativeInterfaceItems() {
			global $menu, $submenu;
			
			$menu['56.3'] = array( '', 'read', 'separator-premise', '', 'wp-menu-separator' );
			$menu['56.4'] = $menu[950];
			unset($menu[950]);
		}

		function modifyAttachmentFields($form_fields, $post) {
			$html = '<a href="'.$post->guid.'" class="send-to-premise-field">'.__('Use This Image').'</a>';

			$form_fields = $form_fields + array('send-to-premise-field' => array('tr' => '<tr style="display: none;" class="send-to-premise-field"><td></td><td>'.$html.'</td></tr>'));

			return $form_fields;
		}

		function modifyLayoutColumns($columns, $screen) {
			if ($screen == $this->_cached_DesignPageSlug) {
				$columns[$this->_cached_DesignPageSlug] = 2;
			}
			return $columns;
		}

		function modifyPinnedStatus() {
			$this->savePinnedStatus($_POST['pinned'] == 'true');
			exit();
		}

		function modifyTemplateInclude($template) {
			if(is_singular()) {
				global $wp_query;
				$post = $wp_query->get_queried_object();
				if($post->post_type == $this->_type_LandingPage) {
					return trailingslashit($this->getThemeDirectory()).$this->getLandingPageType($post->ID).'.php';
				}
			}

			return $template;
		}

		function modifyThemeRootsTransient($roots) {
			if(!isset($roots['premise'])) {
				return false;
			}
			return $roots;
		}

		function modifyWPTitleForLandingPages($title) {
			global $wp_query;
			if(is_singular() && ($post = $wp_query->get_queried_object()) && $post->post_type == $this->_type_LandingPage) {
				$seo = $this->getPremiseSeoMeta($post->ID);
				$stitle = trim($seo['title']);
				if(!empty($stitle)) {
					$title = $stitle;
				}
			}

			return $title;
		}

		function notifyOfDuplicate() {
			global $post;
			$duplicate = get_post_meta($post->ID, $this->_metakey_Duplicated, true);
			include('views/misc/duplicate-notice.php');
			delete_post_meta($post->ID, $this->_metakey_Duplicated);
		}

		function outputGoogleWebsiteOptimizerScriptAsAppropriate() {
			global $wp_query;
			if(is_singular()
			&& ($post = $wp_query->get_queried_object())
			&& $post->post_type == $this->_type_LandingPage
			&& ($settings = $this->getSettings()) && ($trackingSettings = $settings['tracking']) && !empty($trackingSettings['account-id'])
			&& ($tracking = $this->getPremiseTrackingMeta($post->ID)) && $tracking['enable-gwo'] == 1
			) {
				$isAbTest = $tracking['ab'] == 1;
				$isAbOriginal = $tracking['ab-original'] == 1;
				$isGoal = $tracking['page-type'] == 'goal';
				$testId = $tracking['test-id'];

				if($isAbTest && $isAbOriginal) {
					include('views/gwo/ab-control.php');
				} elseif (!$isGoal) {
					include('views/gwo/control.php');
				}
			}
		}

		function outputGoogleWebsiteOptimizerFooterScriptAsAppropriate() {
			global $wp_query;
			if(is_singular()
			&& ($post = $wp_query->get_queried_object())
			&& $post->post_type == $this->_type_LandingPage
			&& ($settings = $this->getSettings()) && ($trackingSettings = $settings['tracking']) && !empty($trackingSettings['account-id'])
			&& ($tracking = $this->getPremiseTrackingMeta($post->ID)) && $tracking['enable-gwo'] == 1
			) {
				$isGoal = $tracking['page-type'] == 'goal';
				$clicksAsConversions = $tracking['link-click-conversion'];
				$testId = $tracking['test-id'];
				$accountId = $trackingSettings['account-id'];
				
				$pattern = '/^([0-9]+)$/';
				$pattern2 = '/^(UA-)?([0-9]+)(-([0-9]))?$/';
				if(preg_match($pattern, $accountId, $matches)) {
					$accountId = "UA-{$accountId}-2";
				} elseif(preg_match($pattern2, $accountId, $matches)) {
					if(!isset($matches[3])) {
						$accountId = "{$accountId}-2";
					}
				}

				if(!$isGoal) {
					include('views/gwo/tracking.php');
					if($clicksAsConversions) {
						include('views/gwo/click-tracking.php');
					}
				} else {
					include('views/gwo/conversion.php');
				}
			}
		}

		function outputLandingPageTypeColumn($column, $postId) {
			switch($column) {
				case 'landing-page-type':
					$name = $this->getLandingPageTypeName($postId);
					esc_html_e($name);
					break;
			}
		}

		function outputMediaButton() {
			global $post;
			if($post->post_type == $this->_type_LandingPage) {
				printf('<a href="%s" class="%s thickbox" title="%s"><img src="%s" alt="%s" /></a>', esc_url(premise_get_media_upload_src('premise-resources-graphics')), 'add_premise_resources', __('Premise Graphics'), plugins_url('resources/images/icon-desat.gif', __FILE__), __('Premise Graphics'));
			}
		}

		function outputMetaTagsAsAppropriate() {
			global $wp_query;
			if(is_singular() && ($post = $wp_query->get_queried_object()) && $post->post_type == $this->_type_LandingPage) {
				remove_action('wp_head', 'rsd_link' );
				remove_action('wp_head', 'wlwmanifest_link' );
				remove_action('wp_head', 'index_rel_link' );
				remove_action('wp_head', 'parent_post_rel_link', 10, 0 );
				remove_action('wp_head', 'start_post_rel_link', 10, 0 );
				remove_action('wp_head', 'adjacent_posts_rel_link_wp_head', 10, 0);
				remove_action('wp_head', 'noindex', 1 );
				remove_action('wp_head', 'wp_generator' );
				remove_action('wp_head', 'rel_canonical' );

				$seo = $this->getPremiseSeoMeta($post->ID);
				if(1 == $seo['disable-feed']) {
					remove_action('wp_head', 'feed_links', 2);
					remove_action('wp_head', 'feed_links_extra', 3);
				}

				include('views/misc/meta-tags.php');
			}
		}

		function outputVisualWebsiteOptimizerScriptAsAppropriate() {
			$settings = $this->getSettings();
			if(isset($settings['tracking']['vwo-account-id']) && !empty($settings['tracking']['vwo-account-id'])) {
				$accountId = $settings['tracking']['vwo-account-id'];
				include('views/vwo/tracking.php');
			}
		}

		function overridePluginsAPIResults($res, $action, $args) {
			if('plugin_information' == $action && $this->_data_PluginSlug == $args->slug) {
				$settings = $this->getSEttings();
				$url = add_query_arg(array('apikey' => $settings['main']['api-key']), $this->_data_PluginInfoURL);
				$response = wp_remote_get($url);

				if(!is_wp_error($response) && 200 == wp_remote_retrieve_response_code($response)) {
					$res = json_decode(wp_remote_retrieve_body($response));
					$sections = array();
					if(is_string($res->sections)) {
						$res->sections = array('Description' => $res->sections);
					} elseif(is_object($res->sections)) {
						foreach($res->sections as $name => $content) {
							$sections[$name] = $content;
						}

						$res->sections = $sections;
					}
				}
			}

			return $res;
		}

		function possiblyChangeThemeConstants() {
			/** Super Hack so we can tell whether we should register the Prose theme as our chosen theme or not **/

			if(is_admin()) { return; }

			global $wp, $wp_filter;
			$parse_request = $wp_filter['parse_request'];
			$request = $wp_filter['request'];

			$wp_filter['parse_request'] = array();
			$wp_filter['request'] = array();
			$wp->parse_request();
			$wp_filter['parse_request'] = $parse_request;
			$wp_filter['request'] = $request;

			$front = get_option('show_on_front');
			$id = get_option('page_on_front');
			if('page' == $front) {
				$post = get_post($id);
			}
			
			if(!empty($wp->query_vars['landing_page']) || ($wp->query_vars['post_type'] == $this->_type_LandingPage) || (!empty($post) && $post->post_type == $this->_type_LandingPage && empty($wp->query_vars))) {
				$this->registerThemeLocation();
				add_filter('stylesheet', array(&$this, 'returnPremise'));;
				add_filter('template', array(&$this, 'returnPremise'));;
				define('BP_DISABLE_ADMIN_BAR', true);
			} 
			
			if('page' == $front && !empty($id) && $post->post_type == $this->_type_LandingPage && empty($wp->query_vars)) {
				$this->_cached_FrontPageId = $post->ID;
 				remove_action('template_redirect', 'redirect_canonical');
				add_action('parse_request', array($this, 'pushLandingPageToFront'));
			}
		}

		function possiblyEnqueueAdministrativeResourcesForAdminPages($hook) {
			if('media-upload-popup' == $hook) {
				$this->enqueueAdministrativeResourcesForAdminPages();
				include('views/misc/media-upload-popup-script.php');
			}
		}

		function possiblyEndOutputBuffering() {
			global $pagenow;
			if($pagenow == 'post-new.php' && $_GET['post_type'] == $this->_type_LandingPage) {
				$result = ob_get_clean();
				include('views/interceptions/post-new.php');
			}
		}

		function possiblyStartOutputBuffering() {
			global $pagenow;
			if($pagenow == 'post-new.php' && $_GET['post_type'] == $this->_type_LandingPage) {
				ob_start();
			}
		}

		function premiseButtonOutput($atts, $content = null) {
			$atts = array_map('trim', $atts);
			$id = $atts['id'];
			
			if(!empty($atts['href'])) {
				$code = '<a href="'.$atts['href'].'" class="premise-button-%s">%s</a>';
			} else {
				$code = '<span class="premise-button-%s">%s</span>';
			}
			
			return sprintf($code, $id, $content);
		}

		function premiseInternalShortcode($atts, $content = null) {
			if(empty($content)) {
				return '';
			} elseif(empty($atts['id'])) {
				return '';
			} else {
				return sprintf('<a href="%s">%s</a>', add_query_arg(array('section' => $atts['id'])), $content);
			}
		}

		function processMailingListContactSubmissions($wp) {
			$data = stripslashes_deep($_POST);

			if(isset($data['constant-contact']) && is_array($data['constant-contact']) && is_numeric($data['constant-contact']['list']) && wp_verify_nonce($data['constant-contact-signup-nonce'], 'constant-contact-signup-'.$data['constant-contact']['list'])) {
				$cc = array_map('trim', $data['constant-contact']);

				$errors = array();
				if(empty($cc['first-name'])) {
					$errors[] = __('Please enter your first name');
				}

				if(empty($cc['last-name'])) {
					$errors[] = __('Please enter your last name');
				}

				if(empty($cc['email'])) {
					$errors[] = __('Please enter your email');
				} elseif(!is_email($cc['email'])) {
					$errors[] = __('Email address is invalid');
				}

				$messages = array();
				if(!empty($errors)) {
					foreach($errors as $body) {
						$messages[] = array('type' => 'error', 'body' => $body);
					}
				} else {
					$result = $this->signupUserForConstantContact($cc['first-name'], $cc['last-name'], $cc['email'], $cc['list']);
				}

				if(true === $result) {
					$messages[] = array('type' => 'update', 'body' => __('You have been successfully subscribed'));
				} elseif(is_array($result) && isset($result['error'])) {
					$messages[] = array('type' => 'error', 'body' => $result['error']);
				}

				set_transient($this->_optin_ConstantContactMessages, $messages);
				wp_redirect(trailingslashit(home_url($cc['currenturl'])).'#'.$cc['formkey']);
				exit();
			}

			if(isset($data['mailchimp']) && is_array($data['mailchimp']) && isset($data['mailchimp']['list']) && wp_verify_nonce($data['mailchimp-signup-nonce'], 'mailchimp-signup-'.$data['mailchimp']['list'])) {
				$mc = array_map('trim', $data['mailchimp']);

				$mv = $this->getMailChimpMergeVars($mc['list']);

				$newvars = array();
				$errors = array();
				foreach($mv as $data) {
					if(empty($mc[$data['tag']]) && $data['req'] == 1) {
						$errors[] = sprintf(__('%s is required'), $data['name']);
					} elseif($data['field_type'] == 'email' && !is_email($mc[$data['tag']])) {
						$errors[] = sprintf(__('%s must be a valid email address'), $data['name']);
					} else {
						$newvars[$data['tag']] = $mc[$data['tag']];
					}
				}

				$messages = array();
				if(!empty($errors)) {
					foreach($errors as $body) {
						$messages[] = array('type' => 'error', 'body' => $body);
					}
				} else {
					$result = $this->signupUserForMailChimp($newvars, $mc['list']);
				}

				if(true === $result) {
					$messages[] = array('type' => 'update', 'body' => __('You have been successfully subscribed'));
				} elseif(is_array($result) && isset($result['error'])) {
					$messages[] = array('type' => 'error', 'body' => $result['error']);
				}

				set_transient($this->_optin_MailChimpMessages, $messages);
				wp_redirect(trailingslashit(home_url($mc['currenturl'])).'#'.$mc['formkey']);
				exit();
			}
		}

		/**
		 * Intercepts values in the administrative interface to do various actions such as:
		 * - Saving settings
		 * - Setting the landing page type for a particular landing page
		 *
		 * @return void
		 */
		function processSubmissions() {
			if(isset($_GET['landing_page_set']) && $_GET['landing_page_set'] == 1 && wp_verify_nonce($_GET['_wpnonce'], 'landing_page_set')) {
				$this->setDefaultLandingPageMeta($_GET['post']);
				$this->setLandingPageType($_GET['post'], $_GET['landing_page']);
				$this->setLandingPageAdviceType($_GET['post'], $_GET['landing_page_advice']);
				wp_redirect(get_edit_post_link($_GET['post'], 'raw'));
				exit();
			}
			if(isset($_POST['save-premise-settings']) && wp_verify_nonce($_POST['save-premise-settings-nonce'],'save-premise-settings')) {
				$this->processPremiseSettings();
				wp_redirect(admin_url('admin.php?page='.$_GET['page'].'&updated=true&premise-settings-updated=true'));
				exit();
			}
			if((isset($_POST['premise-design'])) && wp_verify_nonce($_POST['save-premise-design-settings-nonce'], 'save-premise-design-settings')) {
				$message = $this->processPremiseDesignSettings();
				wp_redirect(admin_url('admin.php?page=premise-style-settings&'.$message));
				exit();
			}
			if(isset($_GET['premise-design-key']) && $_GET['premise-delete-style'] == 'true' && check_admin_referer('premise-delete-style')) {
				$this->deleteConfiguredStyle($_GET['premise-design-key']); 
				wp_redirect(admin_url('admin.php?page=premise-styles&deleted=true'));
				exit();
			} elseif(isset($_GET['premise-duplicate-style']) && $_GET['premise-duplicate-style'] == 'true' && $this->isValidStyleKey($_GET['premise-design-key']) && check_admin_referer('premise-duplicate-style')) {
				$style = $this->getConfiguredStyle($_GET['premise-design-key']);
				$style['premise_style_title'] .= (' - '.__('Copy'));
				$style['premise_style_timesaved'] = current_time('timestamp');
				$this->saveDesignSettings($style, null);
				
				wp_redirect(admin_url('admin.php?page=premise-styles&duplicated=true'));
				exit();
			} elseif(isset($_GET['premise-button-id']) && $_GET['premise-delete-button'] == 'true' && check_admin_referer('premise-delete-button')) {
				$this->deleteConfiguredButton($_GET['premise-button-id']);
				wp_redirect(admin_url('admin.php?page=premise-styles&deleted=true'));
				exit();
			} elseif(isset($_GET['premise-button-id']) && $_GET['premise-duplicate-button'] == 'true' && check_admin_referer('premise-duplicate-button')) {
				$this->duplicateConfiguredButton($_GET['premise-button-id']);
				wp_redirect(admin_url('admin.php?page=premise-styles&duplicated=true'));
				exit();
			}

			$data = stripslashes_deep($_POST);
			if(isset($data['premise-support-submit-request']) && isset($data['premise-support-submit-request-nonce']) && wp_verify_nonce($data['premise-support-submit-request-nonce'], 'premise-support-submit-request')) {
					
				$support = $data['premise-support'];
				
				$PremiseSupport = new Premise_Support;
				$PremiseSupport->sendSupportRequest($support['your-name'], $support['your-email'], $support['your-problem']);
				
				wp_redirect(add_query_arg(array('plugin_status' => $data['plugin_status'], 'paged' => $data['paged'], 'premise-sent-support-request' => 'yes'), admin_url('plugins.php')));
				exit();
			}
		}

		function processDuplicateDisplay() {
			global $pagenow, $post;
			if($pagenow == 'post.php' && $post->post_type == $this->_type_LandingPage) {
				$duplicated = get_post_meta($post->ID, $this->_metakey_Duplicated, true);
				if($duplicated) {
					add_action('admin_notices', array(&$this, 'notifyOfDuplicate'));
				}

				include('views/misc/scribe-override.php');
			}
		}

		function pushLandingPageToFront($wp) {
			if(!empty($this->_cached_FrontPageId)) {
				$wp->query_vars['p'] = $this->_cached_FrontPageId;
				$wp->query_vars['post_type'] = $this->_type_LandingPage;	
			}
		}

		function redirectToPremiseHelpPage() {
			wp_redirect('https://members.getpremise.com/help.aspx');
			exit;
		}

		/**
		 * Registers the landing page type such that administrative users can create and edit them and frontend visitors can view
		 * them at the appropriate URLs.
		 * @return void
		 */
		function registerCustomContentTypes() {
			$settings = $this->getSettings();
			$labels = array(
				'name' => __('Landing Pages'),
				'singular_name' => __('Landing Page'),
				'add_new' => __('Add New'),
				'add_new_item' => __('Add New Landing Page'),
				'edit_item' => __('Edit Landing Page'),
				'new_item' => __('New Landing Page'),
				'view_item' => __('View Landing Page'),
				'search_items' => __('Search Landing Pages'),
				'not_found' => __('No landing pages found'),
				'not_found_in_trash' => __('No landing pages found in Trash'),
				'parent_item_colon' => null
			);
			$args = array(
				'labels' => $labels,
				'description' =>__('Landing pages are designed to increase conversions for your product.  Create them quickly and easily.'),
				'publicly_queryable' => true,
				'exclude_from_search' => true,
				'capability_type' => 'page',
				'hierarchical' => false,
				'public' => true,
				'rewrite' => false,
				'query_var' => true,
				'supports' => array('title','editor','revisions'),
				'register_meta_box_cb' => array(&$this, 'registerLandingPageMetaBoxes'),
				'taxonomies' => array(),
				'show_ui' => true,
				'menu_position' => 950,
				'menu_icon' => plugins_url('resources/images/icon.gif', __FILE__),
				'permalink_epmask' => EP_PAGES,
				'can_export' => true,
				'show_in_nav_menus' => true
			);
			register_post_type($this->_type_LandingPage, $args);
		}

		function retrieveOptInCode() {
			$settings = $this->getSettings();
			$results = array('error' => true, 'code' => '');
			if(!empty($settings['main']['opt-in-code'])) {
				$results = array('error' => false, 'code' => $settings['main']['opt-in-code']);
			}
			echo json_encode($results);
			exit();
		}

		function retrievePageTypeSampleCopy() {
			$copy = $this->retrieveSampleCopyForPage($_POST['id']);

			if(empty($copy)) {
				$results = array('error' => true, 'copy' => '', 'error_message' => __('Could not retrieve sample copy for this page.  Please check your Premise API key.'));
			} else {
				$results = array('error' => false, 'copy' => $copy);
			}

			echo json_encode($results);
			exit();
		}

		function returnPremise($stylesheet) {
			return 'premise';
		}

		function registerShortcodes() {
			add_shortcode('gwo-section', array(&$this, 'gwoSectionShortcode'));
			add_shortcode('aweber-optin-form', array(&$this, 'aweberFormShortcode'));
			add_shortcode('constant-contact-optin-form', array(&$this, 'constantContactFormShortcode'));
			add_shortcode('mailchimp-optin-form', array(&$this, 'mailchimpFormShortcode'));
			add_shortcode('manual-optin-form', array(&$this, 'manualOptInFormShortcode'));
			add_shortcode('premise-internal', array(&$this, 'premiseInternalShortcode'));
			add_shortcode('premise-button', array(&$this, 'premiseButtonOutput'));
		}

		function registerThemeLocation() {
			register_theme_directory('plugins/'.basename(dirname(__FILE__)).'/themes');
		}

		function registerTinyMCEScriptOutput($version) {
			$this->_data_PreviouslyOutputTinyMCEScriptTags = true;

			return $version;
		}
		
		function saveButtonViaAjax() {
			$data = stripslashes_deep($_POST);
			
			if(isset($data['premise-save-button-nonce']) && wp_verify_nonce($data['premise-save-button-nonce'], 'premise-save-button')) {
				$key = $data['premise-button-id'];
				$button = $data['button-editing'];
				
				$this->saveConfiguredButton($button, $key);
			}
			
			exit;
		}

		function saveLandingPageMeta($postId, $post) {
			if(false === (wp_is_post_autosave($post) || wp_is_post_revision($post)) && $post->post_type == $this->_type_LandingPage) {
				$this->processLandingPageTypeSpecificMeta($postId, $post);
				$this->processPremiseSeoMeta($postId, $post);
				$this->processPremiseTrackingMeta($postId, $post);
				$this->processDuplication($postId, $post);

				$settings = $this->getSettings();
				if(($settings['main']['rewrite-root'] || empty($settings['main']['rewrite']))) {
					flush_rewrite_rules();
				}
			}
		}
		
		function settingsBoxes() {
			add_meta_box('premise-settings-global', __('Global Styles'), 'premise_settings_global', $this->_cached_DesignPageSlug, 'column1');
			add_meta_box('premise-settings-global-links', __('Global Links'), 'premise_settings_global_links', $this->_cached_DesignPageSlug, 'column1');
			add_meta_box('premise-settings-wrap', __('Wrap (content area)'), 'premise_settings_wrap', $this->_cached_DesignPageSlug, 'column1');
			add_meta_box('premise-settings-header', __('Header'), 'premise_settings_header', $this->_cached_DesignPageSlug, 'column1');
			add_meta_box('premise-settings-headline-area', __('Main Headline Area'), 'premise_settings_headline_area', $this->_cached_DesignPageSlug, 'column1');
			add_meta_box('premise-settings-blockquotes', __('Blockquotes'), 'premise_settings_blockquotes', $this->_cached_DesignPageSlug, 'column1');
			add_meta_box('premise-settings-notice', __('Notice Box'), 'premise_settings_notice_box', $this->_cached_DesignPageSlug, 'column1');
			add_meta_box('premise-settings-pricing', __('Pricing'), 'premise_settings_pricing', $this->_cached_DesignPageSlug, 'column1');

			add_meta_box('premise-settings-headline', __('In-Page Headlines'), 'premise_settings_headline', $this->_cached_DesignPageSlug, 'column2');
			add_meta_box('premise-settings-footer', __('Footer'), 'premise_settings_footer', $this->_cached_DesignPageSlug, 'column2');
			add_meta_box('premise-settings-input', __('Input Boxes'), 'premise_settings_input_box', $this->_cached_DesignPageSlug, 'column2');
			add_meta_box('premise-settings-buttons', __('Submit Buttons'), 'premise_settings_buttons', $this->_cached_DesignPageSlug, 'column2');
			add_meta_box('premise-settings-content-scroller', __('Content Scroller'), 'premise_settings_content_scroller', $this->_cached_DesignPageSlug, 'column2');
			add_meta_box('premise-settings-video', __('Video'), 'premise_settings_video', $this->_cached_DesignPageSlug, 'column2');
			add_meta_box('premise-settings-optin', __('Opt In'), 'premise_settings_optin', $this->_cached_DesignPageSlug, 'column2');
			add_meta_box('premise-settings-general', __('General Settings'), 'premise_settings_general', $this->_cached_DesignPageSlug, 'column2');
		}

		function useGraphic() {
			$data = stripslashes_deep($_POST);

			$id = $data['slug'];
			$name = $data['name'];
			$filename = $data['filename'];
			$info = $this->getGraphic($id);

			if(is_wp_error($info)) {
				echo json_encode(array('error'=>true, 'error_message'=> $info->get_error_message()));
			} else {
				$upload = wp_upload_bits($filename, false, $info);
				if(is_wp_error($upload)) {
					echo json_encode(array('error'=>true,'error_message'=>$upload->get_error_message()));
				} elseif(is_array($upload) && isset($upload['error']) && $upload['error']) {
					echo json_encode(array('error' => true, 'error_message' => $upload['error']));
				} else {
					list($iinfo['width'], $iinfo['height']) = getimagesize($upload['file']);
					$iinfo['full_url'] = $upload['url'];
					$iinfo['error'] = false;
					$iinfo['html'] = sprintf('<img src="%s" alt="%s" width="%d" height="%d" />', $iinfo['full_url'], $name, $iinfo['width'], $iinfo['height']);
					echo json_encode($iinfo);
				}
			}
			exit();
		}

		/**
		 * Registers the appropriate meta boxes based on the landing page type.
		 * @return void
		 */
		function registerLandingPageMetaBoxes($post) {
			$type = $this->getLandingPageType($post->ID);
			if(in_array($type, array('content-scroller', 'video', 'opt-in', 'pricing'))) {
				remove_post_type_support($this->_type_LandingPage, 'editor');
			}

			do_action("register_meta_boxes-{$type}.php", $post, $type);

			add_meta_box('premise-assistant', __('Landing Page Assistant'), array(&$this, 'displayPremiseEducationMetaBox'), $this->_type_LandingPage, 'side', 'low');
			add_meta_box('premise-options', __('Premise: Settings For This Page'), array(&$this, 'displayPremiseOptionsMetaBox'), $this->_type_LandingPage, 'normal', 'high');
			add_meta_box('premise-seo', __('Premise: SEO Options For This Page'), array(&$this, 'displayPremiseSeoOptionsMetaBox'), $this->_type_LandingPage, 'normal', 'high');
			add_meta_box('premise-tracking', __('Premise: Testing For This Page'), array(&$this, 'displayPremiseTrackingOptionsMetaBox'), $this->_type_LandingPage, 'normal', 'high');
		}

		/** Register Per Type Meta Boxes **/

		function registerContentScrollerTypeMetaBoxes($post) {
			add_meta_box('premise-content-scroller-controls', __('Tab/Panel Settings'), array(&$this, 'displayPremiseContentScrollerMetaBox'), $this->_type_LandingPage, 'side', 'core');
		}

		function registerLandingTypeMetaBoxes($post) {

		}

		function registerLongCopyTypeMetaBoxes($post) {

		}

		function registerOptInTypeMetaBoxes($post) {
			add_meta_box('premise-opt-in', __('Opt In Copy'), array(&$this, 'displayOptInMetaBox'), $this->_type_LandingPage, 'normal', 'high');
			add_meta_box('premise-opt-in-below-copy', __('Copy Below the Opt In Form'), array(&$this, 'displayOptInBelowCopyMetaBox'), $this->_type_LandingPage, 'normal', 'high');
		}

		function registerPricingTypeMetaBoxes($post) {
			add_meta_box('premise-pricing-content', __('Pricing Content'), array(&$this, 'displayPremisePricingContentMetaBox'), $this->_type_LandingPage, 'normal', 'high');
			add_meta_box('premise-pricing-controls', __('Re-order Price Settings'), array(&$this, 'displayPremisePricingMetaBox'), $this->_type_LandingPage, 'side', 'core');
		}

		function registerVideoTypeMetaBoxes($post) {
			add_meta_box('premise-video-box-copy', __('Video Box Copy'), array(&$this, 'displayVideoCopyMetaBox'), $this->_type_LandingPage, 'normal', 'high');
			add_meta_box('premise-video-box-below-copy', __('Copy Below the Video Box'), array(&$this, 'displayVideoBelowCopyMetaBox'), $this->_type_LandingPage, 'normal', 'high');
		}

		/** Display Per Type Meta Boxes **/

		function displayOptInMetaBox($post) {
			$resourcesUrl = plugins_url('resources/', __FILE__);
			include('views/meta-boxes/page-types/opt-in.php');
		}

		function displayOptInBelowCopyMetaBox($post) {
			include('views/meta-boxes/page-types/opt-in-below.php');
		}

		function displayVideoCopyMetaBox($post) {
			$resourcesUrl = plugins_url('resources/', __FILE__);
			include('views/meta-boxes/page-types/video.php');
		}

		function displayVideoBelowCopyMetaBox($post) {
			include('views/meta-boxes/page-types/video-below.php');
		}

		/// PROCESSING

		function processDuplication($postId, $post) {
			$data = stripslashes_deep($_POST);
			if(!empty($data['premise-duplicate-page'])) {
				remove_action('save_post', array(&$this, 'saveLandingPageMeta'), 10, 2);

				$duplicate = $post;
				$duplicate->ID = '';
				$duplicate->post_title .= __(' Copy');
				$duplicate->post_status = 'draft';
				$duplicateId = wp_insert_post($duplicate);

				update_post_meta($duplicateId, $this->_metakey_LandingPageAdviceType, $this->getLandingPageAdviceType($postId));
				update_post_meta($duplicateId, $this->_metakey_LandingPageType, $this->getLandingPageType($postId));
				update_post_meta($duplicateId, $this->_metakey_SeoSettings, $this->getPremiseSeoMeta($postId));
				update_post_meta($duplicateId, $this->_metakey_Settings, $this->getPremiseMeta($postId));
				update_post_meta($duplicateId, $this->_metakey_TrackingSettings, $this->getPremiseTrackingMeta($postId));

				update_post_meta($postId, $this->_metakey_Duplicated, $duplicateId);
				add_action('save_post', array(&$this, 'saveLandingPageMeta'), 10, 2);
			}
		}

		function processLandingPageTypeSpecificMeta($postId, $post) {
			$data = stripslashes_deep($_POST);

			if(is_array($data['premise']) && wp_verify_nonce($data['save-premise-settings-nonce'], 'save-premise-settings')) {
				$premise = $data['premise'];
				$premise['footer'] = $premise['footer'] == 1 ? 1 : 0;
				$premise['header'] = $premise['header'] == 1 ? 1 : 0;

				if(is_array($premise['content-scrollers'])) {
					$temp = $premise['content-scrollers'];
					$premise['content-scrollers'] = array();
					foreach($premise['content-scrollers-order'] as $key) {
						$premise['content-scrollers'][] = $temp[$key];
					}
				}

				if(is_array($premise['pricing-columns'])) {
					$temp = $premise['pricing-columns'];
					$premise['pricing-columns'] = array();
					foreach($premise['pricing-order'] as $key) {
						$attributes = $temp[$key]['attributes'];
						if(!is_array($attributes)) {
							$attributes = array();
						}
						$attributes = array_values(array_filter(array_map('trim', $attributes)));

						$temp[$key]['attributes'] = $attributes;
						$premise['pricing-columns'][] = $temp[$key];
					}
				}
				
				$this->savePremiseMeta($postId, $premise);
			}
		}

		function processPremiseSeoMeta($postId, $post) {
			$data = stripslashes_deep($_POST);

			if(is_array($data['premise-seo']) && wp_verify_nonce($data['save-premise-seo-settings-nonce'], 'save-premise-seo-settings')) {
				$seo = $data['premise-seo'];
				$seo['noindex'] = $seo['noindex'] == 1 ? 1 : 0;
				$seo['nofollow'] = $seo['nofollow'] == 1 ? 1 : 0;
				$seo['noarchive'] = $seo['noarchive'] == 1 ? 1 : 0;
				$seo['disable-feed'] = $seo['disable-feed'] == 1 ? 1 : 0;

				$this->savePremiseSeoMeta($postId, $seo);
			} elseif($post->post_status == 'auto-draft') {
				$settings = $this->getSettings();
				$seo = $settings['seo'];

				$this->savePremiseSeoMeta($postId, $seo);
			}
		}

		function processPremiseTrackingMeta($postId, $post) {
			$data = stripslashes_deep($_POST);

			if(is_array($data['premise-tracking']) && wp_verify_nonce($data['save-premise-tracking-settings-nonce'], 'save-premise-tracking-settings')) {
				$tracking = $data['premise-tracking'];
				$tracking['enable-gwo'] = $tracking['enable-gwo'] == 1 ? 1 : 0;
				$tracking['ab'] = $tracking['ab'] == 1 ? 1 : 0;
				$tracking['ab-original'] = $tracking['ab-original'] == 1 && $tracking['ab'] == 1 ? 1 : 0;
				$tracking['test-id'] = $this->parseTestIdentifierFromGoogleWebsiteOptimizerEmbedCode($tracking['test-id']);
				$tracking['page-type'] = $tracking['page-type'] == 'goal' ? 'goal' : 'test';
				$tracking['link-click-conversion'] = $tracking['link-click-conversion'] == 1 && $tracking['page-type'] == 'test' ? 1 : 0;

				$this->savePremiseTrackingMeta($postId, $tracking);
			}
		}

		function parseAccountIdentifierFromGoogleWebsiteOptimizerEmbedCode($code) {
			$code = trim($code);
			if(empty($code)) {
				return '';
			}
			
			$pattern = '/UA-([0-9]+)-([0-9])/';
			if(preg_match($pattern, $code, $matches)) {
				$code = $matches[0];
			}
			
			return $code;
		}

		function parseTestIdentifierFromGoogleWebsiteOptimizerEmbedCode($code) {
			$code = trim($code);
			if(empty($code)) {
				return '';
			}

			$pattern = "/var k='(\d*)'/";
			$pattern2 = "/trackPageview', '\/(\d*)\//";
			if(preg_match($pattern, $code, $matches)) {
				$code = $matches[1];
			} elseif(preg_match($pattern2, $code, $matches)) {
				$code = $matches[1];
			}

			return $code;
		}

		/**
		 * Handle posted data and save settings for the Premise plugin.
		 * @return void
		 */
		function processPremiseSettings() {
			$data = stripslashes_deep($_POST);
			$premise = $data['premise'];

			if($premise['seo']['indicator'] == 1) {
				$premise['seo']['noindex'] = $premise['seo']['noindex'] == 1 ? 1 : 0;
				$premise['seo']['nofollow'] = $premise['seo']['nofollow'] == 1 ? 1 : 0;
				$premise['seo']['noarchive'] = $premise['seo']['noarchive'] == 1 ? 1 : 0;
				$premise['seo']['disable-feed'] = $premise['seo']['disable-feed'] == 1 ? 1 : 0;
			}

			if(isset($premise['main']['rewrite'])) {
				if($premise['main']['rewrite-root'] == 1) {
					$premise['main']['rewrite'] = '';
				} else {
					$premise['main']['rewrite'] = empty($premise['main']['rewrite']) ? 'landing' : $premise['main']['rewrite'];
				}
			}

			$oldSettings = $this->getSettings();
			$settings = array_merge($oldSettings, $premise);

			$errors = array();
			if($settings['main']['api-key'] != $oldSettings['main']['api-key']) {
				$old = $this->_data_PremiseApi;
				$this->_data_PremiseApi = new Premise_API($settings['main']['api-key']);
				$check = $this->getGraphicsCategories();
				if(is_wp_error($check)) {
					$errors[] = __('Your new Premise API key could not be confirmed.  Please enter a valid key below.');
					$settings['main']['api-key'] = '';
				}
			}

			if($settings['optin']['aweber-authorization'] != $oldSettings['optin']['aweber-authorization']) {
				$aweberCheck = $this->validateAweberAuthorizationCode($settings['optin']['aweber-authorization']);
				if(is_array($aweberCheck) && !isset($aweberCheck['error'])) {
					$settings['optin']['allowed']['aweber'] = 1;
					$settings['optin']['aweber-account-info'] = $aweberCheck;
				} else {
					$settings['optin']['allowed']['aweber'] = 0;
					$errors[] = $aweberCheck['error'];
					$settings['optin']['aweber-authorization'] = '';
				}
			} else {
				$settings['optin']['aweber-account-info'] = $oldSettings['optin']['aweber-account-info'];
				$settings['optin']['allowed']['aweber'] = $oldSettings['optin']['allowed']['aweber'];
			}

			if($settings['optin']['mailchimp-api'] != $oldSettings['optin']['mailchimp-api']) {
				$mailchimpCheck = $this->validateMailChimpAPIKey($settings['optin']['mailchimp-api']);
				if(true === $mailchimpCheck) {
					$settings['optin']['allowed']['mailchimp'] = 1;
				} else {
					$settings['optin']['allowed']['mailchimp'] = 0;
					$errors[] = $mailchimpCheck['error'];
					$settings['optin']['mailchimp-api'] = '';
				}
			} else {
				$settings['optin']['allowed']['mailchimp'] = $oldSettings['optin']['allowed']['mailchimp'];
			}

			if($settings['optin']['constant-contact-username'] != $oldSettings['optin']['constant-contact-username'] || $settings['optin']['constant-contact-password'] != $oldSettings['optin']['constant-contact-password']) {
				$constantContactCheck = $this->validateConstantContactCredentials($settings['optin']['constant-contact-username'], $settings['optin']['constant-contact-password']);
				if(true === $constantContactCheck) {
					$settings['optin']['allowed']['constant-contact'] = 1;
				} else {
					$settings['optin']['allowed']['constant-contact'] = 0;
					$errors[] = $constantContactCheck['error'];
					$settings['optin']['constant-contact-username'] = '';
					$settings['optin']['constant-contact-password'] = '';
				}
			} else {
				$settings['optin']['allowed']['constant-contact'] = $oldSettings['optin']['allowed']['constant-contact'];
			}
			
			$settings['tracking']['account-id'] = $this->parseAccountIdentifierFromGoogleWebsiteOptimizerEmbedCode($settings['tracking']['account-id']);


			$this->saveSettings($settings);

			set_transient($this->_option_SettingsTransient, array('updates' => array(__('Settings Saved.')), 'errors' => $errors), 10);

			flush_rewrite_rules();
		}

		function processPremiseDesignSettings() {
			$data = stripslashes_deep($_POST);
			$key = $data['premise-design-key'];
			$design = $data['premise-design'];

			if(isset($design['reset'])) {
				$settings = $this->getDefaultDesignSettings();
				$message = 'reset=true';
			} else {
				if(!empty($key)) {
					$settings = $this->getDefaultDesignSettings();
				} else {
					$settings = $this->getConfiguredStyle($key);
				}
				$settings = array_merge($settings, $design);

				if($design['minify_css'] != 'true' || !is_writeable(premise_get_stylesheet_location('file'))) {
					unset($settings['minify_css']);
				}
				$message = 'updated=true';
			}

			$key = $this->saveDesignSettings($settings, $key);

			return "premise-design-key={$key}&{$message}";
		}

		function removeContentFilters($content) {
			global $post;
			if($post->post_type == $this->_type_LandingPage) {
				remove_filter('the_content', 'tm_update', 8);
				remove_filter('the_content', 'shrsb_position_menu');
				
				// SOCIABLE
				remove_filter('the_content', 'sociable_display_hook');
				
				// LINK WITHIN
				remove_action('wp_footer', 'linkwithin_load_widget');
				remove_filter('the_content', 'linkwithin_add_hook');
				
				// SHARE THIS
				remove_filter('the_content', 'st_add_widget');
				remove_filter('the_content', 'jw_share_this_links');
				
				// SHARE BAR
				remove_filter('the_content', 'sharebar_auto');
				
				// DIGG DIGG
				remove_filter('the_content', 'dd_content_hook');
				remove_action('the_content', 'pf_show_link');
				
				// YET ANOTHER RELATED POSTS
				remove_filter('the_content','yarpp_default',1200);

				// WP JETPACK				
				remove_filter( 'the_content', 'sharing_display', 19 );
				remove_filter( 'the_content', 'polldaddy_link', 1 );
				remove_filter( 'the_content_rss', 'polldaddy_link', 1 );
				remove_filter( 'the_content', 'youtube_link', 1 );
    			remove_filter( 'the_content_rss', 'youtube_link', 1 );
				
				// POST FOOTER
				remove_action('the_content', 'add_post_footer');
				
				// OUTBRAIN DISPLAY
				remove_filter('the_content', 'outbrain_display');
				
				// WHAT WOULD SETH GODIN DO
				remove_filter('the_content', 'wwsgd_message_filter');
				
				// ADD TO ANY
				remove_filter('the_content', 'A2A_SHARE_SAVE_add_to_content', 98);
				
				// SOCIABLE
				global $sociable;
				remove_filter('the_content', array(&$sociable, 'content_hook'));
				
				global $wpfblike;
				if(is_object($wpfblike)) {
					remove_action('the_content', array(&$wpfblike, 'get_content_button_before'), $wpfblike->buttonpriority);
					remove_action('the_content', array(&$wpfblike, 'get_content_button_after'), $wpfblike->buttonpriority);					
				}
				
				// ShareBar
				remove_filter('the_content', 'sharebar_auto');
				remove_action('init', 'sharebar_init');
				
				// Facebook Like
				remove_filter('the_content', 'Add_Like_Button');
				
				// WP FB Like
				remove_filter('the_content', 'fblike_out');
				
				// WP Tweet Button
				global $wpTweetButton;
				if(is_object($wpTweetButton)) {
					remove_filter('the_content', array(&$wpTweetButton, 'tw_update'), $wpTweetButton->tw_get_option('tw_hook_prio'));
				}
				
				// Facebook Share (New)
				remove_filter('the_content', 'fb_share');
				
				// Cool Author Box
				remove_action('the_content', 'author_BOX_display');
				
				// Share and Follow
				global $cons_shareFollow;
				if(is_object($cons_shareFollow)) {
					remove_filter('the_content', array(&$cons_shareFollow, 'addContent'), 10);
				}
				
				// Nofollow Reciprocity
				remove_filter('the_content', 'wp_nofollow_reciprocity', 10);
				remove_filter('get_footer', 'wp_nofollow_reciprocity_awareness', 10);
				remove_action('wp_footer', 'wp_nofollow_reciprocity_awareness', 10);
			}

			return $content;
		}

		function removeOtherDepdendenciesFromPremisePage() {
			if(is_singular(array($this->_type_LandingPage))) {
				global $cons_shareFollow;
				if(is_object($cons_shareFollow)) {
					remove_action('wp_head', array(&$cons_shareFollow, 'getCDNcodes'),1);
			        remove_action('wp_head', array(&$cons_shareFollow, 'addHeaderCode'),1);
			        remove_action('wp_head', array(&$cons_shareFollow, 'addHeaderCodeEndBlock'),10);
			        remove_action('wp_footer',array(&$cons_shareFollow, 'show_follow_links'),1);
			        remove_action('wp_head',array(&$cons_shareFollow, 'doAnalytics'),10);
				}
			}
		}

		/// DISPLAY THICKBOX CALLBACKS

		function displayPremiseResourcesThickbox() {
			wp_enqueue_style('media');
			wp_enqueue_style('premise-admin', plugins_url('resources/premise-admin.css',__FILE__), array('farbtastic'));

			wp_enqueue_script('premise-admin', plugins_url('resources/premise-admin.js',__FILE__), array('jquery', 'jquery-ui-sortable', 'farbtastic', 'jquery-form'));

			add_filter('media_upload_tabs', array(&$this, 'getPremiseThickboxTabs'));
			return wp_iframe('premise_thickbox');
		}

		function displayPremiseResourcesThickboxOutput() {
			$page = $_GET['tab'];
			if(empty($page)) {
				$page = $_GET['type'];
			}

			switch($page) {
				case 'premise-resources-graphics':
					include('views/thickbox/graphics.php');
					break;
				case 'premise-resources-education':
					include('views/thickbox/education.php');
					break;
				case 'premise-resources-optin':
					include('views/thickbox/optin.php');
					break;
				case 'premise-button-create':
					include('views/thickbox/button-creation.php');
					break;
				case 'premise-button-usage':
					include('views/thickbox/button-usage.php');
					break;
			}
		}

		/// DISPLAY META BOX CALLBACKS

		function displayPremiseContentScrollerMetaBox($post) {
			include('views/meta-boxes/page-types/content-scroller-controls.php');
		}

		function displayPremiseContentScrollerTabs() {
			global $post;

			$type = $this->getLandingPageType($post->ID);
			if('content-scroller' == $type) {
				echo '<div id="premise-content-scroller-tabs">';
				$scrollers = $this->getContentScrollers($post->ID);
				foreach($scrollers as $key => $scroller) {
					if(is_array($scroller)) {
						include('views/meta-boxes/premise-content-tab.php');
					}
				}
				echo '</div>';
				echo '<script type="text/javascript">blockSave = true;</script>';
			}
		}

		function displayPremiseEducationMetaBox($post) {
			$type = $this->getLandingPageType($post->ID);
			$image = plugins_url('resources/images/icon.gif', __FILE__);

			include('views/meta-boxes/premise-education.php');
		}

		function displayPremiseOptionsMetaBox($post) {
			$resourcesUrl = plugins_url('resources/',__FILE__);
			$type = $this->getLandingPageType($post->ID);

			include('views/meta-boxes/premise-options.php');
		}

		function displayPremisePricingContentMetaBox($post) {
			include('views/meta-boxes/page-types/pricing-content.php');
		}

		function displayPremisePricingMetaBox($post) {
			include('views/meta-boxes/page-types/pricing-controls.php');
		}

		function displayPremisePricingColumns() {
			global $post;

			$type = $this->getLandingPageType($post->ID);
			if('pricing' == $type) {
				echo '<div id="premise-pricing-columns">';
				$columns = $this->getPricingColumns($post->ID);
				foreach($columns as $key => $column) {
					if(is_array($column)) {
						include('views/meta-boxes/premise-pricing-column.php');
					}
				}
				echo '</div>';
				echo '<script type="text/javascript">blockSave = true;</script>';
			}
		}

		function displayPremiseSeoOptionsMetaBox($post) {
			include('views/meta-boxes/premise-seo.php');
		}

		function displayPremiseTrackingOptionsMetaBox($post) {
			include('views/meta-boxes/premise-tracking.php');
		}

		/// OPT IN PROVIDERS

		//// AWEBER

		/**
		 *
		 * @param $code
		 * @return AWeberAPI
		 */
		function initializeAweberApi() {
			require_once('lib/aweber_api/aweber_api.php');
		}

		function getAweberLists() {
			$this->initializeAweberApi();
			$settings = $this->getSettings();

			$info = $settings['optin']['aweber-account-info'];
			$aweber = new AWeberAPI($info['consumer_key'], $info['consumer_secret']);

			try {
				$account = $aweber->getAccount($info['access_key'], $info['access_secret']);
			} catch (AWeberException $e) {
				$account = null;
			}

			$list_web_forms = array();
			if($account) {
				foreach ($account->getWebForms() as $this_webform) {
					$link_parts = explode('/', $this_webform->url);
					$list_id = $link_parts[4];
					if (!array_key_exists($list_id, $list_web_forms)) {
						$list_web_forms[$list_id] = array(
	                    'web_forms' => array(),
	                    'split_tests' => array()
						);
					}
					$list_web_forms[$list_id]['web_forms'][] = $this_webform;
				}
				foreach ($account->getWebFormSplitTests() as $this_webform) {
					$link_parts = explode('/', $this_webform->url);
					$list_id = $link_parts[4];
					if (!array_key_exists($list_id, $list_web_forms)) {
						$list_web_forms[$list_id] = array(
	                    'web_forms' => array(),
	                    'split_tests' => array()
						);
					}
					$list_web_forms[$list_id]['split_tests'][] = $this_webform;
				}

				$lists = $account->lists;
				foreach ($lists as $this_list) {
					if (array_key_exists($this_list->id, $list_web_forms)) {
						$list_web_forms[$this_list->id]['list'] = $this_list;
					}
				}
			}

			$return = array();
			foreach($list_web_forms as $id => $data) {
				$item = array('id' => $id, 'name' => $data['list']->name, 'forms' => array());
				foreach($data['web_forms'] as $web_form) {
					$item['forms'][] = array('id' => $web_form->id, 'url' => $web_form->url, 'name' => $web_form->name);
				}
				foreach($data['split_tests'] as $split_test) {
					$item['forms'][] = array('id' => $split_test->id, 'url' => $split_test->url, 'name' => sprintf(__('Split Test: %s'), $web_form->name));
				}
				$return[] = $item;
			}
			return $return;
		}

		function validateAweberAuthorizationCode($code) {
			$this->initializeAweberApi();

			try {
				list($consumer_key, $consumer_secret, $access_key, $access_secret) = AWeberAPI::getDataFromAweberID($code);
			} catch (AWeberException $e) {
				list($consumer_key, $consumer_secret, $access_key, $access_secret) = null;
			}

			if(!$access_secret) {
				return array('error' => __('Invalid Aweber authorization code.  Please make sure you entered it correctly.'));
			}

			$aweber = new AWeberAPI($consumer_key, $consumer_secret);

			try {
				$account = $aweber->getAccount($access_key, $access_secret);
			} catch (AWeberResponseError $e) {
				$account = null;
			}

			if(!$account) {
				return array('error' => __('Unable to connect to Aweber account.  Please try again.'));
			}

			return compact('consumer_key', 'consumer_secret', 'access_key', 'access_secret');
		}

		//// CONSTANT CONTACT

		function getConstantContactLists() {
			require_once('lib/constant_contact_api/constant_contact_api.php');

			$settings = $this->getSettings();
			$optin = $settings['optin'];
			$this->setupConstantContactUtility($this->_optin_ConstantContactKey, $optin['constant-contact-username'], $optin['constant-contact-password']);

			$lists = new ListsCollection();
			list($items) = $lists->getLists();

			$return = array();
			foreach($items as $item) {
				$return[] = array('id' => preg_replace('/[^0-9]/', '', (string)$item->getId()), 'name' => (string)$item->getName());
			}
			return array_slice($return, 3);
		}

		function setupConstantContactUtility($apikey, $username, $password, $action = 'ACTION_BY_CONTACT') {
			CCUtility::$sapikey = $apikey;
			CCUtility::$susername = $username;
			CCUtility::$spassword = $password;
			CCUtility::$saction = $action;
		}

		function signupUserForConstantContact($firstname, $lastname, $email, $list) {
			require_once('lib/constant_contact_api/constant_contact_api.php');

			$settings = $this->getSettings();
			$optin = $settings['optin'];
			$this->setupConstantContactUtility($this->_optin_ConstantContactKey, $optin['constant-contact-username'], $optin['constant-contact-password']);

			$collection = new ContactsCollection();
			list($search) = $collection->searchByEmail($email);
			if(!empty($search)) {
				foreach($search as $possible) {
					if($email == $possible->getEmailAddress()) {
						$contact = $collection->getContact($possible->getLink());
						break;
					}
				}
			}

			$listKey = 'http://api.constantcontact.com/ws/customers/'.$optin['constant-contact-username'].'/lists/'.$list;
			if($contact) {
				$existingLists = $contact->getLists();

				if(in_array($listKey, $existingLists)) {
					return array('error' => __('You have already subscribed to this mailing list'));
				} else {
					$contact->setLists($listKey);
				}
				$contact->setFirstName($firstname);
				$contact->setLastName($lastname);
				$result = $collection->updateContact($contact->getId(), $contact);
			} else {
				$contact = new Contact();
				$contact->setFirstName($firstname);
				$contact->setLastName($lastname);
				$contact->setEmailAddress($email);
				$contact->setLists($listKey);

				$result = $collection->createContact($contact);
			}

			$first = substr((string)$result,0,1);

			if($first == 2) {
				return true;
			} else {
				return array('error' => __('Could not subscribe you to this mailing list'));
			}
		}

		function validateConstantContactCredentials($username, $password) {
			require_once('lib/constant_contact_api/constant_contact_api.php');

			$this->setupConstantContactUtility($this->_optin_ConstantContactKey, $username, $password);

			$utility = new CCUtility();
			$result = $utility->ping();

			if(!empty($result['error'])) {
				return array('error' => __('Invalid Constant Contact credentials.'));
			}

			return true;
		}

		//// MAILCHIMP

		function getMailChimpLists() {
			require_once('lib/mailchimp_api/MCAPI.class.php');

			$settings = $this->getSettings();
			$mailchimp = new MCAPI($settings['optin']['mailchimp-api']);
			$data = $mailchimp->lists(array(), 0, 100);

			$lists = array();
			if(is_array($data) && is_array($data['data'])) {
				foreach($data['data'] as $item) {
					$lists[] = array('id' => $item['id'], 'name' => $item['name']);
				}
			}
			return $lists;
		}

		function getMailChimpMergeVars($id) {
			require_once('lib/mailchimp_api/MCAPI.class.php');

			$settings = $this->getSettings();
			$mailchimp = new MCAPI($settings['optin']['mailchimp-api']);

			if(!isset($this->_optin_MailChimpMergeVars[$id])) {
				$vars = get_transient('mailchimp_merge_vars_'.$id);
				if(!is_array($vars)) {
					$vars = $mailchimp->listMergeVars($id);
					set_transient('mailchimp_merge_vars_'.$id, $vars, time() + 24*60*60);
				}
				$this->_optin_MailChimpMergeVars[$id] = $vars;
			}

			return $this->_optin_MailChimpMergeVars[$id];
		}

		function signupUserForMailChimp($vars, $list) {
			require_once('lib/mailchimp_api/MCAPI.class.php');

			$settings = $this->getSettings();
			$mailchimp = new MCAPI($settings['optin']['mailchimp-api']);

			$result = $mailchimp->listSubscribe($list, $vars['EMAIL'], $vars);

			if(1 == $result) {
				return true;
			} else {
				return array('error' => __('Unable to subscribe you to the list.'));
			}
		}

		function validateMailChimpAPIKey($key) {
			require_once('lib/mailchimp_api/MCAPI.class.php');

			$mailchimp = new MCAPI($key);
			if(!$mailchimp->ping()) {
				return array('error' => __('Invalid MailChimp API key.'));
			}

			return true;
		}

		/// API DELEGATES

		function retrieveAdviceForPage($postId) {
			if(empty($postId)) {
				global $post;
				$postId = $post->ID;
			}

			$type = $this->convertLandingPageToId($this->getLandingPageAdviceType($postId));
			return $this->getAdvice($type);
			return array('advice' => $copy, 'section' => $section);
		}

		function retrieveAdviceSectionsForPage($postId) {
			if(empty($postId)) {
				global $post;
				$postId = $post->ID;
			}

			$advice = $this->getLandingPageAdviceType($postId);
			$content = $this->getAdviceSections($advice);

			return $content;
		}

		function retrieveSampleCopyForPage($postId) {
			if(empty($postId)) {
				global $post;
				$postId = $post->ID;
			}

			$content = $this->getSampleContent($this->convertLandingPageToId($this->getLandingPageAdviceType($postId)));
			if(is_wp_error($content)) {
				$copy = '';
			} else {
				$copy = $content['content'];
			}

			return $copy;
		}

		//// ADVICE

		function getAdviceSections($type) {
			return $this->_data_PremiseApi->getAdviceSections($type);
		}

		function getAdvice($type) {
			return $this->_data_PremiseApi->getAdvice($type);
		}

		function getSampleContent($type) {
			return $this->_data_PremiseApi->getSampleContent($type);
		}

		function getSingleAdvice($section) {
			return $this->_data_PremiseApi->getSingleAdvice($section);
		}

		//// GRAPHICS

		function getGraphicsCategories() {
			return $this->_data_PremiseApi->getGraphicCategories();
		}

		function getGraphics($limit = 10, $page = 1, $category = '', $search = '') {
			return $this->_data_PremiseApi->getGraphics($limit, $page, $category, $search);
		}

		function getGraphic($id) {
			return $this->_data_PremiseApi->getGraphic($id);
		}

		/// DISPLAY SETTINGS CALLBACKS

		function displayPremiseSettingsPage() {
			$messages = get_transient($this->_option_SettingsTransient);
			delete_transient($this->_option_SettingsTransient);
			include('views/settings/header.php');
			switch($_GET['page']) {
				case 'premise-styles':
					include('views/settings/styles.php');
					break;
				case 'premise-style-settings':
					include('views/settings/design.php');
					break;
				case 'premise-main':
				default:
					include('views/settings/main.php');
					break;
			}
			include('views/settings/footer.php');
		}

		/// LOCATIONS

		function getCssDirectory() {
			$info = wp_upload_dir();
			return trailingslashit($info['basedir']).'premise/';
		}

		function getCssUrl() {
			$info = wp_upload_dir();
			return trailingslashit($info['baseurl']).'premise/';
		}

		function getThemeDirectory() {
			return trailingslashit(dirname(__FILE__)).'themes/premise/';
		}

		/// UTILITY

		function getLandingPageUris() {
			global $wpdb;
			$uris = $wpdb->get_col($wpdb->prepare("SELECT post_name FROM {$wpdb->posts} WHERE post_type = %s", $this->_type_LandingPage));
			$uris = array_filter($uris);
			return $uris;
		}

		function getVersionInfo($cache = true) {
			$raw = get_transient($this->_data_VersionTransientName);
			if(!$cache) {
				$raw = false;
			}

			if(!$raw){
				$settings = $this->getSettings();
				$url = add_query_arg(array('apikey' => $settings['main']['api-key']), $this->_data_VersionURL);
				$raw = wp_remote_get($url);

				set_transient($this->_data_VersionTransientName, $raw, 12*60*60);
			}

			if (is_wp_error($raw)) {
				return false;
			} elseif(200 != wp_remote_retrieve_response_code($raw)) {
				return array('is_valid' => 0, 'version' => '', 'url '=> '');
			} else {
				$data = json_decode(wp_remote_retrieve_body($raw));
				return $data;
			}
		}

		/// META

		function setDefaultLandingPageMeta($postId) {
			if(empty($postId)) {
				global $post;
				$postId = $post->ID;
			}

			$settings = $this->getSettings();
			$defaults = array('header-image'=>$settings['main']['default-header-image'], 'header-copy' => $settings['main']['default-header-tagline'], 'footer-copy' => $settings['main']['default-footer-text']);
			$this->savePremiseMeta($postId, $defaults);
		}

		function getPremiseMeta($postId) {
			if(empty($postId)) {
				global $post;
				$postId = $post->ID;
			}

			$settings = wp_cache_get($this->_metakey_Settings, $postId);
			if(!$settings || !is_array($settings)) {
				$settings = get_post_meta($postId, $this->_metakey_Settings, true);
				if(!is_array($settings)) {
					$settings = array();
				}
				wp_cache_set($this->_metakey_Settings, $settings, $postId, time() + 24*60*60);
			}
			return $settings;
		}

		function savePremiseMeta($postId, $settings) {
			if(is_array($settings)) {
				update_post_meta($postId, $this->_metakey_Settings, $settings);
				wp_cache_set($this->_metakey_Settings, $settings, $postId, time() + 24*60*60);
			}
		}

		function getPremiseSeoMeta($postId) {
			if(empty($postId)) {
				global $post;
				$postId = $post->ID;
			}

			$settings = wp_cache_get($this->_metakey_SeoSettings, $postId);
			if(!$settings || !is_array($settings)) {
				$settings = get_post_meta($postId, $this->_metakey_SeoSettings, true);
				if(!is_array($settings)) {
					$settings = array();
				}
				wp_cache_set($this->_metakey_SeoSettings, $settings, $postId, time() + 24*60*60);
			}
			return $settings;
		}

		function savePremiseSeoMeta($postId, $settings) {
			if(is_array($settings)) {
				update_post_meta($postId, $this->_metakey_SeoSettings, $settings);
				wp_cache_set($this->_metakey_SeoSettings, $settings, $postId, time() + 24*60*60);
			}
		}

		function getPremiseTrackingMeta($postId) {
			if(empty($postId)) {
				global $post;
				$postId = $post->ID;
			}

			$settings = wp_cache_get($this->_metakey_TrackingSettings, $postId);
			if(!$settings || !is_array($settings)) {
				$settings = get_post_meta($postId, $this->_metakey_TrackingSettings, true);
				if(!is_array($settings)) {
					$settings = array();
				}
				wp_cache_set($this->_metakey_TrackingSettings, $settings, $postId, time() + 24*60*60);
			}
			return $settings;
		}

		function savePremiseTrackingMeta($postId, $settings) {
			if(is_array($settings)) {
				update_post_meta($postId, $this->_metakey_TrackingSettings, $settings);
				wp_cache_set($this->_metakey_TrackingSettings, $settings, $postId, time() + 24*60*60);
			}
		}

		/// SETTINGS

		function getSettings() {
			$settings = wp_cache_get($this->_metakey_Settings);
			if(!$settings || !is_array($settings)) {
				$settings = get_option($this->_metakey_Settings, array());
				if(!is_array($settings)) {
					$settings = array();
				}
				if(empty($settings['main']['rewrite'])) {
					$settings['main']['rewrite'] = 'landing';
				}
				if(!is_array($settings['optin']['allowed'])) {
					$settings['option']['allowed'] = array();
				}


				wp_cache_set($this->_metakey_Settings, $settings);
			}
			return $settings;
		}

		function saveSettings($settings) {
			if(is_array($settings)) {
				update_option($this->_metakey_Settings, $settings);
				wp_cache_set($this->_metakey_Settings, $settings, null, time() + 24*60*60);
			}
		}

		function getDefaultDesignSettings() {
			$defaults = array( // define our defaults
			'premise_style_title' => __('My Style'),

			##################### globals
			'body_background_color' => '#f3f3f3',
			'body_background_image' => '',
			'body_background_repeat' => 'no-repeat',
			'body_background_align' => 'center',
			'body_font_color' => '#222222',
			'body_font_family' => "Arial, Helvetica, sans-serif",
			'body_font_size' => '14',
			'body_line_height' => '24',

			##################### global links
			'body_link_color' => '#035492',
			'body_link_decoration' => 'underline',
			'body_link_hover_background' => '#ffffff',
			'body_link_hover_background_select' => 'hex',
			'body_link_hover' => '#035492',
			'body_link_hover_decoration' => 'none',

			##################### wrap
			'wrap_width' => '800',
			'wrap_background_color' => '#ffffff',
	        'wrap_background_color_select' => 'hex',
			'wrap_margin_top' => "15",
			'wrap_margin_bottom' => "0",
			'wrap_padding' => "20",
			'wrap_border' => "5",
			'wrap_border_color' => '#eeeeee',
			'wrap_border_style' => 'solid',
			'wrap_corner_radius' => "5",
			'wrap_background_shadow' => "none",

			##################### header
			'header_image_height' => '230',
			'header_background_color' => '#ffffff',
	        'header_background_color_select' => 'hex',

			##################### headline area
			'headline_area_background_color' => '#ffffff',
			'headline_area_background_color_select' => 'hex',
			'headline_area_background_image' => '',
			'headline_area_background_repeat' => 'no-repeat',
			'headline_area_background_align' => 'center',
			'headline_area_margin_top' => '1',
			'headline_area_margin_bottom' => '30',
			'headline_area_padding' => '0',
			'headline_area_align' => 'center',

			##################### headline area title
			'headline_title_font_color' => '#222222',
			'headline_title_font_family' => "Arial, Helvetica, sans-serif",
			'headline_title_font_size' => '40',
			'headline_title_font_style' => 'normal',
			'headline_title_font_weight' => 'bold',

			##################### headline area subtitle
			'headline_subtitle_font_color' => '#555555',
			'headline_subtitle_font_family' => "Arial, Helvetica, sans-serif",
			'headline_subtitle_font_size' => '22',
			'headline_subtitle_font_style' => 'normal',
			'headline_subtitle_font_weight' => 'normal',

			##################### blockquotes
	        'blockquotes_background_color' => '#f3f3f3',
	        'blockquotes_background_color_select' => 'hex',
			'blockquotes_padding_topbottom' => '30',
			'blockquotes_padding_leftright' => '15',
	  		'blockquotes_font_color' => '#333333',
			'blockquotes_font_family' => "Georgia, serif",
			'blockquotes_font_size' => '16',
			'blockquotes_font_style' => 'italic',
	        'blockquotes_border' => "1",
			'blockquotes_border_color' => '#dddddd',
			'blockquotes_border_style' => 'solid',

			##################### notice box
	        'notice_background_color' => '#f9f2d6',
	        'notice_background_color_select' => 'hex',
	  		'notice_font_color' => '#48310e',
			'notice_font_family' => "Georgia, serif",
			'notice_font_size' => '16',
			'notice_font_style' => 'italic',
	        'notice_border' => "1",
			'notice_border_color' => '#eada9c',
			'notice_border_style' => 'solid',
			'notice_padding_topbottom' => '15',
			'notice_padding_leftright' => '25',

			##################### headlines
			'headline_font_family' => "Georgia, serif",
			'h1_font_size' => '26',
			'h2_font_size' => '22',
			'h3_font_size' => '20',
			'h4_font_size' => '18',
			'h1_font_color' => '#222222',
			'h2_font_color' => '#222222',
			'h3_font_color' => '#222222',
			'h4_font_color' => '#222222',
			'headline_font_style' => 'normal',
			'headline_font_weight' => 'normal',
			'headline_text_align' => 'left',
			'headline_text_transform' => 'none',

			##################### headline links
			'h2_link_color' => '#222222',
			'h2_link_decoration' => 'none',
			'h2_link_hover' => '#444444',
			'h2_link_hover_decoration' => 'none',

			##################### footer
			'footer_font_color' => '#555555',
			'footer_font_size' => '11',
			'footer_font_weight' => 'normal',
			'footer_text_align' => 'center',
			'footer_text_transform' => 'none',

			##################### footer links
			'footer_link_color' => '#555555',
			'footer_link_decoration' => 'none',
			'footer_link_hover' => '#555555',
			'footer_link_hover_decoration' => 'underline',

			##################### input box
	        'input_background_color' => '#f3f3f3',
	        'input_background_color_select' => 'hex',
	  		'input_font_color' => '#666666',
			'input_font_family' => "Arial, Helvetica, sans-serif",
			'input_font_style' => 'normal',
	        'input_border' => "1",
			'input_border_color' => '#dddddd',
			'input_border_style' => 'solid',

			##################### buttons
			'button_background_color' => '#444444',
	        'button_background_color_select' => 'hex',
			'button_background_hover_color' => '#222222',
	        'button_background_hover_color_select' => 'hex',
			'button_font_color' => '#ffffff',
			'button_font_family' => "Arial, Helvetica, sans-serif",
	        'button_font_size' => '13',
			'button_border' => "1",
			'button_border_color' => '#dddddd',
			'button_border_style' => 'solid',
			'button_border_radius' => '5',
			'button_padding' => '5',
			'button_text_transform' => 'uppercase',

			##################### Content Scroller

			'content_scroller_background_color' => '#f3f3f3',
	        'content_scroller_background_color_select' => 'hex',
			'content_scroller_color' => '#222222',
			'content_scroller_padding' => '15',
			'content_scroller_arrows_background' => '#035492' ,
			'content_scroller_arrows_background_select' => 'hex',
			'content_scroller_arrows_color' => '#ffffff',
			'content_scroller_tabs_background_color' => '#e5e5e5',
			'content_scroller_tabs_background_color_select' => 'hex',
			'content_scroller_tabs_color' => '#222222',
			'content_scroller_tabs_active_background_color' => '#035492',
			'content_scroller_tabs_active_background_color_select' => 'hex',
			'content_scroller_tabs_active_color' => '#ffffff',
			'content_scroller_tabs_radius' => '10',

			##################### Video Holder
			'video_holder_background' => '#f3f3f3',
			'video_holder_background_select' => 'hex',
			'video_holder_color' => '#222222',
	        'video_holder_border' => "1",
			'video_holder_border_color' => '#dddddd',
			'video_holder_border_style' => 'solid',
			'video_holder_radius' => '0',
			'video_holder_padding' => '20',

			##################### Opt In Holder
			'optin_holder_background' => '#f3f3f3',
			'optin_holder_background_select' => 'hex',
			'optin_holder_color' => '#222222',
	        'optin_holder_border' => "1",
			'optin_holder_border_color' => '#dddddd',
			'optin_holder_border_style' => 'solid',
			'optin_holder_radius' => '0',
			'optin_holder_padding' => '20',

			##################### Opt In Container
			'optin_form_background' => '#ffffff',
			'optin_form_background_select' => 'hex',
			'optin_form_padding' => '1',
			'optin_form_radius' => '0',
	        'optin_form_border' => "1",
			'optin_form_border_color' => '#dddddd',
			'optin_form_border_style' => 'solid',

			'optin_form_header_background' => '#035492',
			'optin_form_header_background_select' => 'hex',
			'optin_form_header_font' => "'Palatino Linotype', 'Book Antiqua', Palatino, serif",
			'optin_form_header_font_size' => '24',
			'optin_form_header_color' => '#ffffff',
			'optin_form_header_padding' => '10',

			'optin_form_contents_background' => '#ffffff',
			'optin_form_contents_background_select' => 'hex',
			'optin_form_contents_color' => '#222222',
			'optin_form_contents_padding' => '15',

			'optin_form_contents_submit_background' => '#4d8a05',
			'optin_form_contents_submit_background_select' => 'hex',
			'optin_form_contents_submit_background_hover' => '#b82300',
			'optin_form_contents_submit_background_hover_select' => 'hex',
			'optin_form_contents_submit_color' => '#ffffff',
			'optin_form_contents_submit_padding' => '9',

			##################### Pricing Table
			'pricing_tier_background' => '#ffffff',
			'pricing_tier_background_select' => 'hex',
			'pricing_tier_border' => '1',
			'pricing_tier_border_color' => '#dddddd',
			'pricing_tier_border_style' => 'solid',
			'pricing_tier_border_radius' => '0',
			'pricing_tier_padding' => '1',

			'pricing_tier_header_background' => '#035492',
			'pricing_tier_header_background_select' => 'hex',
			'pricing_tier_header_color' => '#ffffff',
			'pricing_tier_header_align' => 'center',
			'pricing_tier_header_padding' => '15',
			'pricing_tier_header_font' => "Georgia, serif",
			'pricing_tier_header_font_size' => '22',
			'pricing_tier_header_font_style' => 'normal',
			'pricing_tier_header_font_weight' => 'normal',

			'pricing_tier_features_background' => '#f3f3f3',
			'pricing_tier_features_background_select' => 'hex',
			'pricing_tier_features_color' => '#222222',
			'pricing_tier_features_padding' => '15',

			'pricing_tier_cta_background' => '#4d8a05',
			'pricing_tier_cta_background_select' => 'hex',
			'pricing_tier_cta_background_hover' => '#eb7028',
			'pricing_tier_cta_background_hover_select' => 'hex',
			'pricing_tier_cta_color' => '#ffffff',
			'pricing_tier_cta_padding' => '9',
			'pricing_tier_cta_align' => 'center',

			##################### install flag (do not edit)
			'installed' => 'true',

			##################### general settings
			'minify_css' => ''
			);

			return apply_filters('premise_settings_defaults', $defaults);
		}

		function getDesignSettings() {
			$settings = get_option($this->_option_DesignSettings, $this->getDefaultDesignSettings());
			if(!is_array($settings)) {
				$settings = $this->getDefaultDesignSettings();
			}
				
			if(isset($settings['body_background_color'])) {
				$settings['premise_style_timesaved'] = current_time('timestamp');
				$settings['premise_style_title'] = __('Default');
				$settings = array(0 => $settings);
				update_option($this->_option_DesignSettings, $settings);
			}
			
			return $settings;
		}

		function saveDesignSettings($settings, $key) {
			if(is_array($settings)) {
				$existing = $this->getDesignSettings();
				$settings['premise_style_timesaved'] = current_time('timestamp');
				if(isset($existing[$key])) {
					$existing[$key] = $settings;
				} else {
					$existing[] = $settings;
					$key = array_pop(array_keys($existing));
				}
				
				update_option($this->_option_DesignSettings, $existing);
			}
			return $key;
		}

		function getPinnedStatus() {
			$status = wp_cache_get($this->_option_PinnedStatus);
			if(false === $status) {
				$status = get_option($this->_option_PinnedStatus, 0);
				wp_cache_set($this->_option_PinnedStatus, $status);
			}
			return $status == 1;
		}

		function savePinnedStatus($status) {
			update_option($this->_option_PinnedStatus, $status);
			wp_cache_set($this->_option_PinnedStatus, $status, null, time() + 24*60*60);
		}

		/// LANDING PAGE UTILITY

		function getAvailableLandingPageTypes() {
			return $this->_data_RegisteredLandingPages;
		}

		function getLandingPageInformation($info) {
			if(empty($info)) {
				return false;
			}
			$nameMatches = preg_match('|Template Name:(.*)$|mi', $info, $name);
			$descriptionMatches = preg_match('|Template Description:(.*)$|mi', $info, $description);

			$name = $name[1];
			$description = $description[1];

			if(empty($name)) {
				return false;
			}

			return array('name'=>$name, 'description'=>$description);
		}

		function getLandingPageAdviceType($postId) {
			if(empty($postId)) {
				global $post;
				$postId = $post->ID;
			}

			$meta = get_post_meta($postId, $this->_metakey_LandingPageAdviceType, true);
			if(empty($meta) || is_array($meta)) {
				$meta = $this->getLandingPageType($postId);
				$meta = str_replace('.php', '', $meta);
			}
			return $meta;
		}

		function getLandingPageType($postId) {
			if(empty($postId)) {
				global $post;
				$postId = $post->ID;
			}

			$meta = get_post_meta($postId, $this->_metakey_LandingPageType, true);
			return $meta;
		}

		function setLandingPageType($postId, $type) {
			$post = get_post($postId);
			$types = $this->getAvailableLandingPageTypes();

			$template = false;
			foreach($types as $atype) {
				if($atype['template'] == $type) {
					$template = true;
				}
			}

			$keys = array_keys($types);
			if($post->post_type == $this->_type_LandingPage && $template) {
				update_post_meta($postId, $this->_metakey_LandingPageType, $type);
				update_post_meta($postId, '_wp_page_template', $type);
			}
		}

		function setLandingPageAdviceType($postId, $type) {
			$post = get_post($postId);
			$types = $this->getAvailableLandingPageTypes();
			$keys = array_keys($types);

			if($post->post_type == $this->_type_LandingPage && in_array($type, $keys)) {
				update_post_meta($postId, $this->_metakey_LandingPageAdviceType, $type);
			}
		}

		function getLandingPageTypeName($postId) {
			$type = $this->getLandingPageAdviceType($postId);
			$available = $this->getAvailableLandingPageTypes();
			if(isset($available[$type])) {
				return trim($available[$type]['name']);
			} else {
				return _e('Unknown');
			}
		}

		/// STYLESHEET UTILITY

		/// STYLES UTILITY

		function isValidStyleKey($key) {
			$settings = $this->getDesignSettings();
			return isset($settings[$key]);
		}
		
		function deleteConfiguredStyle($key) {
			$settings = $this->getDesignSettings();
			unset($settings[$key]);
			update_option($this->_option_DesignSettings, $settings);
		}

		function getConfiguredStyle($key = null) {
			$styles = $this->getDesignSettings();
			
			if(null === $key) {
				global $premise_style_configuration_key;
				$key = $premise_style_configuration_key;
			}
			
			if(isset($styles[$key])) {
				$style = $styles[$key];
			} else {
				global $premise_style_configuration_should_use_defaults;
				if($premise_style_configuration_should_use_defaults) {
					$style = $this->getDefaultDesignSettings();
				} else {
					$style = array_shift($styles);
				}
			}
			
			return $style;
		}
		
		/// BUTTONS UTILITY
		
		function deleteConfiguredButton($id) {
			$buttons = $this->getConfiguredButtons($id);
			unset($buttons[$id]);
			$this->saveConfiguredButtons($buttons);
		}
		
		function duplicateConfiguredButton($id) {
			$button = $this->getConfiguredButton($id);
			$copy = $button;
			$button['title'] .= ' - '.__('Copy');
			$this->saveConfiguredButton($button, null);
		}
		
		function getButtonCode($button, $key = null) {
			ob_start();
			include('views/misc/button-code.php');
			return ob_get_clean();
		}
		
		function getConfiguredButtons() {
			$buttons = get_option($this->_option_ConfiguredButtons, array());
			if(!is_array($buttons)) {
				$buttons = array();
			}
			return $buttons;
		}
		
		function getConfiguredButton($key) {
			$buttons = $this->getConfiguredButtons();
			
			if(isset($buttons[$key])) {
				return $buttons[$key];
			} else {
				return $this->getDefaultButtonConfiguration();
			}
		}
		
		function getDefaultButtonConfiguration() {
			return array(
				'title' => __('Default'),
				'font-family' => 'inherit',
				'font-size' => 14,
				'font-color' => '#123d54',
				'padding-tb' => 10,
				'padding-lr' => 20,
				'background-color-1' => '#afd9fa',
				'background-color-2' => '#afd9fa',
				'background-color-2-position' => 25,
				'background-color-3' => '#ffffff',
				'background-color-3-position' => 50,
				'background-color-4' => '#588fad',
				'background-color-4-position' => 75,
				'background-color-5' => '#588fad',
				'background-color-hover-1' => '#588fad',
				'background-color-hover-2' => '#588fad',
				'background-color-hover-2-position' => 25,
				'background-color-hover-3' => '#ffffff',
				'background-color-hover-3-position' => 50,
				'background-color-hover-4' => '#afd9fa',
				'background-color-hover-4-position' => 75,
				'background-color-hover-5' => '#afd9fa',
				'border-radius' => 6,
				'border-width' => 1,
				'border-color' => '#003366',
				'drop-shadow-x' => 0,
				'drop-shadow-y' => 1,
				'drop-shadow-size' => 3,
				'drop-shadow-color' => '#000000',
				'drop-shadow-opacity' => 0.5,
				'inset-shadow-x' => 0,
				'inset-shadow-y' => 0,
				'inset-shadow-size' => 1,
				'inset-shadow-color' => '#ffffff',
				'inset-shadow-opacity' => 1,
				'text-shadow-1-x' => 0,
				'text-shadow-1-y' => -1,
				'text-shadow-1-size' => 0,
				'text-shadow-1-color' => '#000000',
				'text-shadow-1-opacity' => .7,
				'text-shadow-2-x' => 0,
				'text-shadow-2-y' => 1,
				'text-shadow-2-size' => 0,
				'text-shadow-2-color' => '#ffffff',
				'text-shadow-2-opacity' => .3,
			);
		}
		
		function getRgbForHex($color){
		    $color = str_replace('#', '', $color);
		    if (strlen($color) != 6){ return array(0,0,0); }
		    $rgb = array();
		    for ($x=0;$x<3;$x++){
		        $rgb[$x] = hexdec(substr($color,(2*$x),2));
		    }
		    return implode(',',$rgb);
		}
		
		function saveConfiguredButton($button, $key) {
			$button['lastsaved'] = current_time('timestamp');
			$buttons = $this->getConfiguredButtons();
			
			if(isset($key)) {
				$buttons[$key] = $button;
			} else {
				do {
					$key = wp_generate_password(12, false, false);
				} while(isset($buttons[$key]));
				$buttons[$key] = $button;
			}
			
			$this->saveConfiguredButtons($buttons);
		}
		
		function saveConfiguredButtons($buttons) {
			if(is_array($buttons)) {
				update_option($this->_option_ConfiguredButtons, $buttons);
			}
		}
		
		function saveConfiguredButtonsStylesheet($oldvalue, $newvalue) {
			$css = '';
			if(is_array($newvalue)) {
				foreach($newvalue as $key => $button) {
					$css .= strip_tags($this->getButtonCode($button, $key));
				}
			}
			$css = premise_minify_css($css);
				
		    $handle = @fopen(premise_get_custom_buttons_stylesheet_path(), 'w');
		    @fwrite($handle, $css);
		    @fclose($handle);
		}
		
		function saveCustomButtonsCss() {
			$buttons = $this->getConfiguredButtons();
		}
		
		/// TEMPLATE TAG DELEGATES

		function theEditor($content, $id = 'content', $prev_id = 'title', $media_buttons = true, $tab_index = 2, $quicktags_before = false) {
			$number = $this->_data_EditorNumber;
			$this->_data_EditorNumber++;
			include('views/misc/editor.php');
		}

		/// LANDING PAGE TEMPLATE TAG DELEGATES

		//// GENERAL

		function getFooterCopy($postId) {
			$meta = $this->getPremiseMeta($postId);
			$copy = trim($meta['footer-copy']);
			if(empty($copy)) {
				$settings = $this->getSettings();
				$copy = trim($settings['main']['default-footer-text']);
			}

			return $copy;
		}

		function getHeaderCopy($postId) {
			$meta = $this->getPremiseMeta($postId);

			return $meta['header-copy'];
		}

		function getHeaderImage($postId) {
			$meta = $this->getPremiseMeta($postId);
			$image = trim($meta['header-image']);
			if(empty($image)) {
				$settings = $this->getSettings();
				$image = trim($settings['main']['default-header-image']);
			}

			return $image;
		}

		function shouldHaveFooter($postId) {
			$meta = $this->getPremiseMeta($postId);

			return 1 != $meta['footer'];
		}

		function shouldHaveHeader($postId) {
			$meta = $this->getPremiseMeta($postId);

			return 1 != $meta['header'];
		}
		
		function shouldHaveHeaderImage($postId) {
			$meta = $this->getPremiseMeta($postId);
			
			return 1 != $meta['header-image-hide'];
		}

		//// CONTENT SCROLLER

		function getContentScrollers($postId) {
			$meta = $this->getPremiseMeta($postId);

			$scrollers = $meta['content-scrollers'];
			if(empty($scrollers) || !is_array($scrollers)) {
				$scrollers = array(array('title'=>__('Tab 1'), 'text'=>''));
			}
			return $scrollers;
		}

		function getContentScrollerShowTabs($postId) {
			$meta = $this->getPremiseMeta($postId);

			return $meta['show-arrows'] == 'tabs' || $meta['show-arrows'] == '' || $meta['show-arrows'] == 'arrows-and-tabs';
		}

		function getContentScrollerShowArrows($postId) {
			$meta = $this->getPremiseMeta($postId);

			return $meta['show-arrows'] == '' || $meta['show-arrows'] == 'arrows-and-tabs';
		}

		//// Sales Page

		function getSubhead($postId) {
			$meta = $this->getPremiseMeta($postId);

			return $meta['subhead'];
		}

		//// OPT-IN

		function getOptinAlign($postId) {
			$meta = $this->getPremiseMeta($postId);

			if(empty($meta['optin-placement'])) {
				$meta['optin-placement'] = 'left';
			}

			return $meta['optin-placement'];
		}

		function getOptinCopy($postId) {
			$meta = $this->getPremiseMeta($postId);

			return $meta['optin-copy'];
		}

		function getOptinBelowCopy($postId) {
			$meta = $this->getPremiseMeta($postId);

			return $meta['below-optin-copy'];
		}

		function getOptinFormCode($postId) {
			$meta = $this->getPremiseMeta($postId);

			return $meta['optin-form-code'];
		}

		//// VIDEO

		function getVideoAlign($postId) {
			$meta = $this->getPremiseMeta($postId);

			if(empty($meta['video-placement'])) {
				$meta['video-placement'] = 'left';
			}

			return $meta['video-placement'];
		}

		function getVideoCopy($postId) {
			$meta = $this->getPremiseMeta($postId);

			return $meta['video-copy'];
		}

		function getVideoBelowCopy($postId) {
			$meta = $this->getPremiseMeta($postId);

			return $meta['below-video-copy'];
		}

		function getVideoEmbedCode($postId) {
			$meta = $this->getPremiseMeta($postId);

			return $meta['video-embed-code'];
		}

		function getVideoImage($postId) {
			$meta = $this->getPremiseMeta($postId);

			return $meta['video-image'];
		}

		function getVideoImageTitle($postId) {
			$meta = $this->getPremiseMeta($postId);

			return $meta['video-image-title'];
		}

		//// PRICING

		function getAbovePricingTableContent($postId) {
			$meta = $this->getPremiseMeta($postId);
			return $meta['above-pricing-table-copy'];
		}

		function getBelowPricingTableContent($postId) {
			$meta = $this->getPremiseMeta($postId);
			return $meta['below-pricing-table-copy'];
		}

		function getPricingBulletMarker($postId) {
			$meta = $this->getPremiseMeta($postId);
			return $meta['pricing-bullets'];
		}

		function getPricingBulletColor($postId) {
			$meta = $this->getPremiseMeta($postId);
			return $meta['pricing-bullets-color'];
		}

		function getPricingColumns($postId) {
			$meta = $this->getPremiseMeta($postId);

			$columns = $meta['pricing-columns'];
			if(empty($columns) || !is_array($columns)) {
				$columns = array(array('title'=>__('Basic Plan'), 'attributes'=>array('')));
			}
			return $columns;
		}

	}

	include('lib/api-loader.php');
	global $Premise;
	$Premise = new Premise;

	include('lib/premise-support.php');
	include('lib/template-tags.php');
	include('lib/design-settings.php');
	include('lib/design-settings-support.php');
	include('lib/export.php');
	include('lib/import.php');
	include('lib/stylesheets.php');

	function premise_plugins_url($path) {
		return plugins_url($path, __FILE__);
	}
}
